#include <Arduino.h>
#include <UTFT.h>
#include <UTouch.h>
#include <BaseComp.h>
#include <BaseTemps.h>

//UTFT_M myGLCD(ILI9341_16, 38, 39, 40, 41); // Selon le modèle d'écran
UTFT_M myGLCD(ITDB32S, 38, 39, 40, 41); // Selon le modèle d'écran
UTouch myTouch(6, 5, 4, 3, 2);

extern uint8_t TinyFont[];

unsigned long dureeInactif = 0; // Durée d'inactivité de la veille
int scrDimX;
int scrDimY;
int clFond = 0; // couleur du fond d'écran
byte fontDimX = 0;
byte fontDimY = 0;

unsigned int oaAct[0xC0];
unsigned int faAct[0xC0];
unsigned int fi_oaAct[0xC0];
unsigned int fa_oiAct[0xC0];

int bmp24To16(long bmp24) { // transformation d'une couleur au format 888 vers le format 565
  return  (0xF800 & (bmp24 >> 8)) | (0x07E0 & (bmp24 >> 5)) | (0x001F & (bmp24 >> 3));
}

byte ongletActif = 0; // si le composant Pages n'est pas utilisé, c'est l'onglet indice 0 qui est pris par d�faut

UTFT_M::UTFT_M(byte model, int RS, int WR, int CS, int RST, int SER) : UTFT(model, RS, WR, CS, RST, SER) { // constructeur de UTFT_M
} // extension de la classe UTFT pour y ajouter la méthode "drawImg"

void UTFT_M::drawImg(int x, int y, int sx, int sy, bitmapdatatype data) // dessine une image à partir d'un fichier en SRAM et non en Flash
{
  int tx, ty, tc;
  if (orient == PORTRAIT) {
    cbi(P_CS, B_CS);
    setXY(x, y, x + sx - 1, y + sy - 1);
    for (tc = 0; tc < sx * sy; tc++)
      LCD_Write_DATA(data[tc] >> 8, data[tc] & 0xFF);
    sbi(P_CS, B_CS);
  }
  else {
    cbi(P_CS, B_CS);
    for (ty = 0; ty < sy; ty++)	{
      setXY(x, y + ty, x + sx - 1, y + ty);
      for (tx = sx - 1; tx >= 0; tx--)
        LCD_Write_DATA(data[(ty * sx) + tx] >> 8, data[(ty * sx) + tx] & 0xff);
    }
    sbi(P_CS, B_CS);
  }
  clrXY();
}

void UTFT_M::drawTransparentBitmap(int x, int y, int sx, int sy, bitmapdatatype data) {
  unsigned int col;
  int tx, ty, tc;
  if (orient == PORTRAIT) {
    for (tc = 0; tc < (sx * sy); tc++) {
      col = pgm_read_word(&data[tc]);
      if (col != BMP_TRANSPARENT) {
        cbi(P_CS, B_CS);
        setXY(x + tc % sx, y + tc / sx, x + tc % sx, y + tc / sx);
        LCD_Write_DATA(col >> 8, col & 0xff);
        sbi(P_CS, B_CS);
      }
    }
  }
  else	{
    for (ty = 0; ty < sy; ty++)
      for (tx = sx - 1; tx >= 0; tx--) {
        col = pgm_read_word(&data[(ty * sx) + tx]);
        if (col != BMP_TRANSPARENT) {
          cbi(P_CS, B_CS);
          setXY(x + tx, y + ty, x + tx, y + ty);
          LCD_Write_DATA(col >> 8, col & 0xff);
          sbi(P_CS, B_CS);
        }
      }
  }
  clrXY();
}

void UTFT_M::drawTransparentBitmap(int x, int y, int sx, int sy, bitmapdatatype data, int deg, int rox, int roy) {
  unsigned int col;
  int tc, tx, ty, newx, newy;
  double radian;
  radian = deg * 0.017453292;
  if (orient == PORTRAIT) {
    for (tc = 0; tc < (sx * sy); tc++) {
      col = pgm_read_word(&data[tc]);
      if (col != BMP_TRANSPARENT) {
        cbi(P_CS, B_CS);
        setXY(x + tc % sx, y + tc / sx, x + tc % sx, y + tc / sx);
        LCD_Write_DATA(col >> 8, col & 0xff);
        sbi(P_CS, B_CS);
      }
    }
  }
  else	{
    for (ty = 0; ty < sy; ty++)
      for (tx = sx - 1; tx >= 0; tx--) {
        col = pgm_read_word(&data[(ty * sx) + tx]);
        if (col != BMP_TRANSPARENT) {
          cbi(P_CS, B_CS);
          newx = x + rox + (((tx - rox) * cos(radian)) - ((ty - roy) * sin(radian)));
          newy = y + roy + (((ty - roy) * cos(radian)) + ((tx - rox) * sin(radian)));
          setXY(newx, newy, newx, newy);
          LCD_Write_DATA(col >> 8, col & 0xff);
          sbi(P_CS, B_CS);
        }
      }
  }
  clrXY();
}

void UTFT_M::drawArc(int x, int y, int r, float a1, float a2) {
  // Trace un arc de cercle de rayon "r", centré en "x" et "y" à partir de l'angle "a1" et de longueur "a2" (en °C)
  //	- dans le sens trigonométrique si "a2" est positif,
  //	- dans le sens horaire si "a2" est négatif,
  float nbPt = 2 * PI * r; // nombre de points nécessaires pour tracer un cercle entier
  float d2 = abs(nbPt * a2 / 360.0);
  if (a2 > 0) // tracé dans le sens trigonométrique
    for (int i = 0; i < round(d2); i++) {
      float alpha = 2 * PI * (a1 / 360 + i / nbPt);
      drawPixel(x + r * cos(alpha), y - r * sin(alpha));
    }
  else // tracé dans le sens horaire
    for (int i = 0; i < round(abs(d2)); i++) {
      float alpha = 2 * PI * (a1 / 360 - i / nbPt);
      drawPixel(x + r * cos(alpha), y - r * sin(alpha));
    }
}

int nbComp = 0; // nombre de composants créés
PtrComp * compDeb = NULL; // élément du début de la liste
PtrComp * compCour = new PtrComp; // pointeur d'élément de liste

Clavier * clavier = NULL; // pointeur sur le clavier

boolean creation = true; // à vrai pendant la phase de setup

void defEcran(int HouV, int clF) { // initialisation de l'écran
  creation = true;
  clFond = clF;
  myGLCD.InitLCD(HouV);
  myGLCD.clrScr();
  myGLCD.setBackColor(clF);
  myGLCD.setColor(VGA_BLACK);
  myGLCD.fillScr(clFond);
  myTouch.InitTouch(HouV);
  myTouch.setPrecision(PREC_HI);
  scrDimX = myGLCD.getDisplayXSize();
  scrDimY = myGLCD.getDisplayYSize();
  myGLCD.setFont(TinyFont);
  fontDimX = myGLCD.getFontXsize();
  fontDimY = myGLCD.getFontYsize();
  installeComp(ongletActif);
  creation = false;
}

void installeComp(int noA) {
  PtrComp * compAct = compDeb;
  for (int i = 0; i < nbComp; i++) { // effaçage et affichage automatique des composants visuels au changement d'onglet
    if (creation)
      compAct->ptrB->init(); // initialise les composants pendant le setup
    if (compAct->ptrB->page == noA || compAct->ptrB->page == 0xFF) { // 0xFF -> composant visible/actif sur toutes les pages
      compAct->ptrB->actif = true;
      compAct->ptrB->affiche(); // ... et affiche ceux de l'onglet s�lectionné
    }
    else {
      compAct->ptrB->actif = false;
    }
    compAct = compAct->suiv;
  }
}

void activiteComp(boolean act) {
  PtrComp * compAct = compDeb;
  for (int i = 0; i < nbComp; i++) { // liste des composants visuels
    compAct->ptrB->actif = act; // modifie l'activité de tous les composants
    compAct = compAct->suiv;
  }
}

void scruteEvent() { // scrutation d'évènements
  if (myTouch.dataAvailable()) {
    myTouch.read();
    dureeInactif = millis();
    compCour = compDeb;
    for (int i = 0; i < nbComp; i++) { // boucle qui permet de voir si un composant visuel a été appuyé
      compCour->ptrB->select(myTouch.getX(), myTouch.getY());
      compCour = compCour->suiv;
    }
  }
  courTemps = debTemps; // Voir biblioth�que BaseTemps
  for (int i = 0; i < indTemps; i++) { // boucle qui permet de voir si un composant temps a été activé
    courTemps->ptrT->select(millis());
    courTemps = courTemps->suiv;
  }
  compCour = compDeb;
  for (int i = 0; i < nbComp; i++) { // boucle qui permet de voir si il y a un composant ayant le tag = 0xF0 (composant infoDH par exemple)
    if (compCour->ptrB->tag == 0xF0) {
      compCour->ptrB->select(0, 0);
    }
    compCour = compCour->suiv;
  }
}

/*-----------------------------------------------------*/
/*                         COMPBASE                         */
/*-----------------------------------------------------*/

boolean CompBase::select(int sX, int sY) { // fonction virtuelle
}

void CompBase::init() { // fonction virtuelle
}

void CompBase::affiche(boolean b) { // fonction virtuelle
}

void CompBase::affiche() { // fonction virtuelle
}

void CompBase::efface() { // fonction virtuelle
}

/*-----------------------------------------------------*/
/*                           BOUTON                           */
/*-----------------------------------------------------*/
Bouton::Bouton(int a, int b, int c, int d, String s, bitmapdatatype ic, boolean rh, byte p) {
  tag = 0x01;
  x = a;
  y = b;
  dX = c;
  dY = d;
  exclusif = false;
  retourHaut = rh;
  appuye = false;
  page = p;
  titre = s;
  clTitre = VGA_BLACK;
  icone = ic;
}

boolean Bouton::select(int sX, int sY) {
  if (actif && visible && sX > x && sX < x + dX && sY > y && sY < y + dY) {
    if (valide && !(appuye && exclusif))
      appuye = ! appuye;
    affiche(appuye);
    if (valide) {
      while (myTouch.dataAvailable()) // attend que l'appui sur l'écran cesse
        myTouch.read();
      if (retourHaut) {
        appuye = false;
        affiche(false);
      }
      else {
        if (exclusif) {
          PtrComp * compExc = compDeb;
          for (int i = 0; i < nbComp; i++) { // liste des composants visuels
            if (compExc->ptrB->groupe == groupe && compExc->ptrB != this) {
              compExc->ptrB->appuye = false;
              compExc->ptrB->affiche(false);
            }
            compExc = compExc->suiv;
          }
        }
      }
      if (surAppui != NULL)
        surAppui(appuye);
      return true;
    }
    else
      return false;
  }
  else
    return false;
}

void Bouton::efface() {
  visible = false;
  myGLCD.setColor(clFond);
  myGLCD.fillRect(x, y, x + dX, y + dY);
}

void  Bouton::affiche() {
  affiche(appuye);
}

void Bouton::dessineBtn(int x, int y, String titre, boolean app) {
  myGLCD.setColor(VGA_BLACK);
  myGLCD.drawRoundRect(x, y, x + dX, y + dY); // Entourage
  if (app)
    myGLCD.setColor(BTN_TRAIT);
  else
    myGLCD.setColor(VGA_WHITE);
  myGLCD.drawLine(x + 2, y + 1, x + dX - 2, y + 1); // Ligne horizontale en haut
  myGLCD.drawLine(x + 1, y + 2, x + 1, y + dY - 2); // ligne verticale à gauche
  if (app)
    myGLCD.setColor(VGA_WHITE);
  else
    myGLCD.setColor(BTN_TRAIT);
  myGLCD.drawLine(x + 2, y + dY - 1, x + dX - 2, y + dY - 1); // Ligne horizontale en bas
  myGLCD.drawLine(x + dX - 1, y + 2, x + dX - 1, y + dY - 2); // Ligne verticale à droite
  if (app) {
    myGLCD.setColor(BTN_APPUYE);
    myGLCD.setBackColor(BTN_APPUYE);
  }
  else {
    myGLCD.setColor(BTN_RELACHE);
    myGLCD.setBackColor(BTN_RELACHE);
  }
  myGLCD.fillRoundRect(x + 2, y + 2, x + dX - 2, y + dY - 2);
  myGLCD.setColor(clTitre);
  if (icone != NULL) {
    myGLCD.print(titre, x + 20 + (dX - 20 - titre.length()*fontDimX) / 2, y + (dY - fontDimY) / 2);
    myGLCD.drawTransparentBitmap(x + 4, y + (dY - 16) / 2, 16, 16, icone); // les icones des boutons doivent être de dimension 16 x 16
  }
  else
    myGLCD.print(titre, x + (dX - titre.length()*fontDimX) / 2, y + (dY - fontDimY) / 2);
}

void Bouton::affiche(boolean app) { // app = true -> bouton appuyé
  if (visible)
    dessineBtn(x, y, titre, app);
}

/*-----------------------------------------------------*/
/*                           EDITEUR                           */
/*-----------------------------------------------------*/

Editeur::Editeur(int a, int b, int c, int d, String s, byte ts, char al, boolean ls, byte p) {
  tag = 0x02;
  x = a;
  y = b;
  dX = c;
  dY = d;
  posCaret = 0;
  posAlign = ALIGN_GAUCHE;
  clEditeur = VGA_WHITE; // couleur du fond de texte
  clTexte = VGA_BLACK; // pour l'entourage
  typeSaisie = ts;
  debTexte = s;
  finTexte = "";
  texte = debTexte + finTexte;
  align = al; // alignement
  lectureSeule = ls;
  page = p;
  maj = false;
  surQuitter = NULL;
  if (clavier == NULL) // si aucun clavier n'a été créé, on en crée un
    clavier = new Clavier;
}

void Editeur::addChar(String b) { // limite la longueur du texte à "n" caractères
  if (texte.length() < dX / fontDimX - 1) // le nombre de caractères est limité à la largeur de l'écran
    debTexte = debTexte + b;
  affiche();
}

void Editeur::majValeur(String t) {
}

void Editeur::affiche(String t) {
  debTexte = t;
  finTexte = "";
  affiche();
}

void Editeur::affiche() {
  visible = true;
  texte = debTexte + finTexte;
  if (actif) {
    myGLCD.setColor(VGA_BLACK); // couleur de trait
    myGLCD.drawRect(x, y, x + dX, y + dY); // rectangle d'entourage
    myGLCD.setColor(clEditeur); // couleur de remplissage
    myGLCD.fillRect(x + 1, y + 1, x + dX - 1, y + dY - 1); // remplissage de l'intérieur ou effaçage de qui y était déjà
    myGLCD.setBackColor(clEditeur); // couleur de fond de texte
    myGLCD.setColor(clTexte); // Couleur du texte
    switch (align) {
      case 'G': // cadrage à gauche
        posAlign = ALIGN_GAUCHE;
        break;
      case 'C': // cadrage au milieu
        posAlign = (dX - texte.length() * fontDimX) / 2;
        break;
      case 'D': // cadrage à droite
        posAlign = dX - (texte.length() + 0.5) * fontDimX;
        break;
    }
    myGLCD.print(texte, x + posAlign, y + (dY - fontDimY) / 2);
    if ( ! lectureSeule)
      posCaret = placeCaret(debTexte.length() * fontDimX);
  }
}

int Editeur::placeCaret(int p) { // p est la longueur de texte à partir de son d�but
  int pos;
  if (p > texte.length()*fontDimX) {
    pos = texte.length() * fontDimX;
    debTexte = debTexte + finTexte;
    finTexte = "";
  }
  else {
    pos = (p / fontDimX) * fontDimX;
    debTexte = texte.substring(0, p / fontDimX);
    finTexte = texte.substring(p / fontDimX);
  }
  myGLCD.setColor(clEditeur); // couleur de remplissage
  if (posCaret != 0)
    myGLCD.fillRect(posCaret, y + dY - 8, posCaret + 5, y + dY - 8 + 7); // posCaret = position du Caret par rapport au bord gauche de l'écran
  myGLCD.drawTransparentBitmap(x + posAlign + pos - 3, y + dY - 8, 5, 8, Caret);
  return x + posAlign + pos - 3;
}

void Editeur::efface() {
  visible = false;
  myGLCD.setColor(clFond); // couleur de fond d'écran
  myGLCD.fillRect(x, y, x + dX, y + dY);
}

boolean Editeur::select(int sX, int sY) {
  if (actif && visible && sX > x && sX < x + dX && sY > y && sY < y + dY) {
    if ( ! lectureSeule) { // mode édition de texte
      if ( ! clavier->visible) { // on rend le clavier visible  pour édition de texte
        maj = true;
        clEditeur = VGA_AQUA;
        affiche();
        activiteComp(false); // désactive tous les composants ...
        actif = true; // ... sauf lui-même
        clavier->actif = true;
        clavier->ptrEdt = this; // on envoie au clavier un pointeur sur l'éditeur qui demande du texte
        if (y + dY + HAUTEUR_CLAVIER + 5 > myGLCD.getDisplayYSize()) // pour ne pas masquer l'éditeur avec le clavier, on place ce dernier où l'éditeur n'est pas
          clavier->montre(y - (HAUTEUR_CLAVIER + 5), typeSaisie);
        else
          clavier->montre(y + dY + 5, typeSaisie);
      }
      posCaret = placeCaret(sX - (x + posAlign) + fontDimX / 2); // mise en place du caret
    }
    if (surAppui != NULL)
      surAppui(lectureSeule); // fonction externe
    return true;
  }
  else
    return false;
}

/*-----------------------------------------------------*/
/*                       BOITELISTE                        */
/*-----------------------------------------------------*/

String premStr(String s, char d, byte * p0) { // extrait d'une chaîne la partie située avant le caractère 'd'
  byte p1 = s.indexOf(d, * p0);
  if (p1 >= 0)
    s = s.substring(* p0, p1);
  * p0 = p1 + 1;
  return s;
}

byte nbChar(String s, char c) { // retourne le nombre de caractères 'c' trouvés dans la chaîne "s"
  byte nb = 0;
  for (byte i = 0; i < s.length(); i++)
    if (s[i] == c)
      nb++;
  return nb;
}

byte posChar(String s, char c, byte n) { // retourne la position du nième caractère 'c' dans la chaîne "s"
  byte p = 0;
  for (byte i = 0; i < s.length(); i++) {
    if (s[i] == c)
      p++;
    if (p == n)
      return i;
  }
  return -1;
}

BoiteListe::BoiteListe(int a, int b, int c, String s, char al, boolean ls, byte p) : Editeur(a, b, c, 24, premStr(s, '&', 0), 0, al, ls, p) {
  tag = 0x10;
  hL = 16; // hauteur d'une ligne d'item = 16 pixels
  hB = 0;
  yH = 0;
  yB = 0;
  listeOuverte = false;
  icone = FlecheB;
  items = s;
  if (items == "") {
    nbItems = 0;
    index = -1;
  }
  else {
    nbItems = nbChar(s, '&') + 1;
    index = 0; // la première valeur de la liste
  }
  choix = Editeur::texte;
}

byte BoiteListe::assigneItems(String s) {
  items = s;
  if (items == "") {
    nbItems = 0;
    index = -1;
  }
  else {
    nbItems = nbChar(s, '&') + 1;
    index = 0; // la première valeur de la liste
  }
  Editeur::affiche(items.substring(0, items.indexOf('&')));
  choix = Editeur::texte;
  return nbItems;
}

void BoiteListe::affiche() {
  Editeur::affiche();
  myGLCD.drawTransparentBitmap(x + dX, y, 21, 25, icone);
}

void BoiteListe::affiche(String t) {
  Editeur::affiche(t);
}

int BoiteListe::hauteurBoite() {
  nbItems = 0;
  for (int i = 0; i < items.length(); i++)
    if ( items[i] == '&') {
      nbItems++;
    }
  if (items.length() > 0)
    nbItems++;
  if (y + dY / 2 < myGLCD.getDisplayYSize() / 2)
    return hL * nbItems; // ouverture vers le bas
  else
    return -hL * nbItems; // ouverture vers le haut
}

byte BoiteListe::ajoutStr(String a, byte p) { // ajoute la chaîne "a" à la cha�ne "s" à une position déterminée par p
  byte nb = 0;
  if (items == "")
    items = a;
  else {
    nb = nbChar(items, '&');
    switch (p) {
      case 0: { // ajout au début de s
          items = a + '&' + items;
          break;
        }
      case 0xFF: { // ajout à la fin de s
          items = items + '&' + a;
          break;
        }
      default: { // insertion entre le début et la fin, à la position p
          if (p < nb) {
            byte pos = posChar(items, '&', p);
            items = items.substring(0, pos) + '&' + a + items.substring(pos);
          }
        }
    }
  }
  if (p == 0) {
    choix = a;
    Editeur::affiche(a);
  }
  nbItems++;
  return nb + 2;
}

byte BoiteListe::supprStr(byte p) { // supprime la chaîne "a" de la chaîne "items" àune position déterminée par p
  byte nb = 0;
  if (items == "")
    return 0;
  else {
    nb = nbChar(items, '&');
    if (p == 0) {
      items = items.substring(items.indexOf('&') + 1);
      choix = items.substring(0, items.indexOf('&'));
    }
    else if (p == nbItems - 1) {
      items = items.substring(0, items.lastIndexOf('&'));
      choix = items.substring(items.lastIndexOf('&') + 1);
    }
    else {
      items = items.substring(0, posChar(items, '&', p)) + items.substring(posChar(items, '&', p + 1));
      index--;
      choix = items.substring(posChar(items, '&', p - 1) + 1, posChar(items, '&', p));
    }
    Editeur::affiche(choix);
  }
  return nb;
}

byte BoiteListe::supprStr(String t) { // supprime la chaîne "t" de la chaîne "items"
  byte p = items.indexOf(t);
  if (p > -1) {
    String str = items.substring(0, p);
    index = nbChar(str, '&');
    return supprStr(index);
  }
  else
    return -1; // pas trouvé
}

void BoiteListe::majValeur(String t) {
  if (nbItems == 0) {
    ajoutStr(Editeur::texte, 0);
  }
  else {
    if (t != "")
      items.replace(choix, t);
    else
      nbItems = supprStr(index);
  }
}

boolean BoiteListe::select(int sX, int sY) {
  if (actif && visible) {
    if ( ! lectureSeule && ! listeOuverte && sX > x && sX < x + dX && sY > y && sY < y + dY) { // mode édition de texte, ouverture du clavier
      if ( ! clavier->visible) { // on rend le clavier visible  pour édition de texte
        clEditeur = VGA_AQUA;
        affiche();
        activiteComp(false); // désactive tous les composants ...
        actif = true; // ... sauf lui-même
        clavier->actif = true;
        clavier->ptrEdt = this; // on envoie au clavier un pointeur sur l'éditeur qui demande du texte
        if (y + dY + HAUTEUR_CLAVIER + 5 > myGLCD.getDisplayYSize()) // pour ne pas masquer l'éditeur avec le clavier, on place ce dernier où l'éditeur n'est pas
          clavier->montre(y - (HAUTEUR_CLAVIER + 5), typeSaisie);
        else
          clavier->montre(y + dY + 5, typeSaisie);
      }
      posCaret = placeCaret(sX - (x + posAlign) + fontDimX / 2); // mise en place du caret
    }
    if ( ! clavier->visible) {
      if (( ! lectureSeule && ! listeOuverte && sX > x + dX && sX < x + dX + 21 && sY > y && sY < y + dY)
          || ((lectureSeule || ( !lectureSeule && listeOuverte)) && sX > x && sX < x + dX + 21 && sY > y + min(0, hB) && sY < y + dY + max(0, hB))) {
        if ( ! listeOuverte) { // dessiner la boîte et son contenu
          hB = hauteurBoite();
          listeOuverte = true;
          if (hB > 0) { // ouverture vers le bas
            yB = y + dY + hB;
            yH = y + dY;
          }
          else { // ouverture vers le haut
            yH = y + hB;
            yB = y;
          }
          myGLCD.setColor(VGA_WHITE);
          myGLCD.fillRect(x, yH, x + dX, yB);
          myGLCD.setColor(VGA_BLACK);
          myGLCD.drawRect(x, yH, x + dX, yB);
          byte p = 0;
          for (byte i = 0; i < nbItems; i++) {
            myGLCD.print(premStr(items, '&', &p), x + 3, yH + 4 + hL * i);
          }
          activiteComp(false); // désactive tous les composants ...
          actif = true; // ... sauf lui-même
        }
        else { // effacer la boîte en réinitialisant l'écran
          boolean fermer = false;
          if (sX > x && sX < x + dX && (sY < y || sY > y + dY)) { // changement de données uniquement si appui hors de la ligne de base
            index = (sY - yH) / hL;
            byte p = 0;
            for (byte i = 0; i < index; i++) {
              premStr(items, '&', &p);
            }
            choix = premStr(items, '&', &p);
            Editeur::affiche(choix);
            fermer = true;
          }
          if (fermer || (sX > x && sX < x + dX + 21 && sY > y && sY < y + dY)) { // referme le boîte
            listeOuverte = false;
            hB = 0;
            yH = 0;
            yB = 0;
            myGLCD.fillScr(clFond);
            activiteComp(true); // réactive tous les composants
            installeComp(ongletActif); // ré-affiche la page en cours
            if (surQuitter != NULL)
              surQuitter(0); // fonction externe
          }
        }
      }
    }
    /*		while (myTouch.dataAvailable()) // attend que l'appui sur l'écran cesse
    			myTouch.read();*/
    if (surAppui != NULL)
      surAppui(lectureSeule); // fonction externe
    return true;
  }
  else
    return false;
}

/*-----------------------------------------------------*/
/*                              LABEL                             */
/*-----------------------------------------------------*/

Label::Label(int a, int b, String t, int c, byte p) {
  tag = 0x03;
  x = a;
  y = b;
  titre = t;
  clTitre = c;
  page = p;
}

void Label::affiche() {
  if (visible) {
    myGLCD.setBackColor(clFond);
    myGLCD.setColor(clTitre);
    myGLCD.print(titre, x, y);
  }
}

void Label::affiche(String t) {
  titre = t;
  affiche();
}

void Label::efface() {
  myGLCD.setColor(clFond);
  myGLCD.fillRect(x, y, x + titre.length()*fontDimX, y + fontDimY);
}

/*-----------------------------------------------------*/
/*                              CADRE                             */
/*-----------------------------------------------------*/

Cadre::Cadre(int a, int b, int c, int d, String t, int ct, int cc, byte p) {
  tag = 0x08;
  x = a;
  y = b;
  dX = c;
  dY = d;
  titre = t;
  clTitre = ct;
  clCadre = cc;
  page = p;
}

void Cadre::affiche() {
  if (visible) {
    myGLCD.setBackColor(clFond);
    myGLCD.setColor(clCadre);
    myGLCD.drawRoundRect(x, y, x + dX, y + dY);
    myGLCD.setColor(clTitre);
    myGLCD.print(" " + titre + " ", x + 10, y - 4);
  }
}

void Cadre::affiche(String t) {
  titre = t;
  affiche();
}

void Cadre::efface() {
  myGLCD.setColor(clFond);
  myGLCD.drawRoundRect(x, y, x + dX, y + dY);
  myGLCD.fillRect(x, y - 4, x + 10 + (titre.length() + 2)*fontDimX, y + fontDimY - 4);
}

/*-----------------------------------------------------*/
/*                        BOITETITRE                      */
/*-----------------------------------------------------*/

BoiteTitre::BoiteTitre(int a, int b, int c, int d, String t, int clL, int clC, byte p) {
  tag = 0x31;
  x = a;
  y = b;
  dX = c;
  dY = d;
  titre = t;
  clLabel = clL;
  clCadre = clC;
  page = p;
}

void BoiteTitre::affiche() {
  myGLCD.setBackColor(clFond);
  if (visible)
    myGLCD.setColor(clCadre);
  else
    myGLCD.setColor(clFond);
  myGLCD.drawRoundRect(x, y, x + dX, y + dY);
  if (visible) {
    myGLCD.setColor(clFond);
    myGLCD.drawLine(x + 10, y, x + titre.length()*fontDimX + 17, y);
    myGLCD.setColor(clLabel);
    myGLCD.print(titre, x + 15, y - 4);
  }
  else
    myGLCD.fillRect(x + 15, y - 4, x + titre.length()*fontDimX + 15, y - 4 + fontDimY);
}

void BoiteTitre::affiche(String t) {
  titre = t;
  affiche();
}

void BoiteTitre::efface() {
  visible = false;
  affiche();
}

/*-----------------------------------------------------*/
/*                           MARQUE                           */
/*-----------------------------------------------------*/

Marque::Marque(int a, int b, String t, int c, byte p) {
  tag = 0x04;
  x = a;
  y = b;
  dY = 13;
  titre = t;
  clMarque = c;
  coche = false;
  page = p;
}

void Marque::affiche() {
  visible = true;
  myGLCD.setBackColor(clFond);
  myGLCD.setColor(clMarque);
  if (coche)
    myGLCD.drawTransparentBitmap(x, y, 13,  13, CheckBox1);
  else
    myGLCD.drawTransparentBitmap(x, y, 13,  13, CheckBox0);
  myGLCD.print(titre, x + 20, y + (dY - fontDimY) / 2);
}

void Marque::efface() {
  visible = false;
  myGLCD.setColor(clFond);
  myGLCD.fillRect(x, y, x + titre.length()*fontDimX, y + fontDimY);
}

boolean Marque::select(int sX, int sY) {
  dX = 20 + titre.length() * fontDimX;
  if (actif && visible && sX > x && sX < x + dX && sY > y && sY < y + dY) {
    coche = !coche;
    affiche();
    while (myTouch.dataAvailable()) // attend que l'appui sur l'écran cesse
      myTouch.read();
    if (surAppui != NULL)
      surAppui(coche); // fonction externe
    return true;
  }
  else
    return false;
}

/*-----------------------------------------------------*/
/*                     BOUTONCHOIX                    */
/*-----------------------------------------------------*/

BoutonChoix::BoutonChoix(int a, int b, String t, int c, byte g, byte i, byte p) {
  tag = 0x05;
  x = a;
  y = b;
  dY = 13;
  titre = t;
  clChoix = c;
  groupe = g;
  index = i;
  coche = false;
  page = p;
}

void BoutonChoix::affiche(boolean b) {
  coche = b;
  affiche();
}

void BoutonChoix::affiche() {
  visible = true;
  myGLCD.setBackColor(clFond);
  myGLCD.setColor(clChoix);
  if (coche)
    myGLCD.drawTransparentBitmap(x, y, 13,  13, BtnRad1);
  else
    myGLCD.drawTransparentBitmap(x, y, 13,  13, BtnRad0);
  myGLCD.print(titre, x + 20, y + (dY - fontDimY) / 2);
}

void BoutonChoix::efface() {
  visible = false;
  myGLCD.setColor(clFond);
  myGLCD.fillRect(x, y, x + titre.length()*fontDimX, y + fontDimY);
}

boolean BoutonChoix::select(int sX, int sY) {
  dX = 20 + titre.length() * fontDimX;
  if (actif && visible && sX > x && sX < x + dX && sY > y && sY < y + dY) {
    if (! coche) {
      coche = true;
      PtrComp * compRad = compDeb;
      for (int i = 0; i < nbComp; i++) { // liste des composants visuels
        if (compRad->ptrB->groupe == groupe && compRad->ptrB != this) {
          compRad->ptrB->affiche(false);
        }
        compRad = compRad->suiv;
      }
    }
    affiche();
    /*		while (myTouch.dataAvailable()) // attend que l'appui sur l'�cran cesse
    			myTouch.read();*/
    if (surAppui != NULL)
      //			surAppui(index, coche); // fonction externe
      surAppui(index, titre); // fonction externe
    return true;
  }
  else
    return false;
}

/*-----------------------------------------------------*/
/*                            CLAVIER                          */
/*-----------------------------------------------------*/

Clavier::Clavier() : Bouton(0, 0, 0, 0, "", NULL, true, 0xFE) {
  tag = 0x11;
  visible = false; // ne sera visible que sur la demande d'un Editeur
  nbC = 0; nbL = 0;	pX = 0;	pY = 0; maG = 0; maB = 0; hC = 0;
  y = 0;
  MajToMinToNb = 0;
  touche = MAJUSCULES;
  for (int i = 0; i < 10; i++)
    ptrBmp[i] = NULL;
  ptrBmp[0] = Min;
  ptrBmp[1] = Espace;
  ptrBmp[8] = Efface;
  ptrBmp[9] = OK;
}

void Clavier::choixSaisie(byte tS) {
  switch (tS) {
    case 0:
      touche = MAJUSCULES;
      break;
    case 1:
      touche = MINUSCULES;
      break;
    case 2:
      touche = CHIFFRES_PONCT;
      break;
  }
}

void Clavier::montre(int h, byte tS) { // rend le clavier visible devant les autres composants
  y = h + 2;
  MajToMinToNb = tS;
  choixSaisie(tS);
  hC = y + 3; // hC = haut du clavier
  myGLCD.setColor(VGA_BLACK);
  myGLCD.drawRect(2, y, myGLCD.getDisplayXSize() - 3, y + HAUTEUR_CLAVIER);
  myGLCD.setColor(VGA_WHITE);
  myGLCD.fillRect(3, y + 1, myGLCD.getDisplayXSize() - 4, y + HAUTEUR_CLAVIER - 1);
  visible = true;
  affiche();
}

void Clavier::affiche() { // affiche les touches du clavier
  if (visible) {
    nbL = 3;
    nbC = 10;
    pX = (myGLCD.getDisplayXSize() - 5) / nbC;
    dX = pX - 3; // 3 = espace inter-touches
    pY = 33;
    dY = pY - 3; // 3 = espace inter-touches
    maG = ALIGN_GAUCHE;
    maB = (dX - 16) / 2;
    for (int j = 0; j < nbL; j++)
      for (int i = 0; i < nbC; i++) {
        dessineBtn(maG + pX * i, hC + pY * j, String(touche[i + nbC * j]), false);
        switch (i + nbC * j) {
          case 20:
            switch (MajToMinToNb) {
              case 0:
                myGLCD.drawTransparentBitmap(maG + maB + pX * i, 2 + hC + pY * j, 16,  28, Min);
                break;
              case 1:
                myGLCD.drawTransparentBitmap(maG + maB + pX * i, 2 + hC + pY * j, 16,  28, NbPonct);
                break;
              case 2:
                myGLCD.drawTransparentBitmap(maG + maB + pX * i, 2 + hC + pY * j, 16,  28, Maj);
                break;
            }
            break;
          case 21: myGLCD.drawTransparentBitmap(maG + maB + pX * i, 6 + hC + pY * j, 16,  16, Espace); break;
          case 28: myGLCD.drawTransparentBitmap(maG + maB + pX * i, 6 + hC + pY * j, 16,  16, Efface); break;
          case 29: myGLCD.drawTransparentBitmap(maG + maB + pX * i, 6 + hC + pY * j, 16,  16, OK); break;
        }
      }
  }
}

void Clavier::dessinTouche(int i, int j) {
  if (i + nbC * j - 20 >= 0 && ptrBmp[i + nbC * j - 20] != NULL)
    myGLCD.drawTransparentBitmap(maG + maB + pX * i, 6 + hC + pY * j, 16,  16, ptrBmp[i + nbC * j - 20]);
}

boolean Clavier::select(int sX, int sY) {
  int eX, eY;
  if (actif && visible) {
    for (int j = 0; j < nbL; j++)
      for (int i = 0; i < nbC; i++) {
        eX = maG + (pX) * i;
        eY = hC + (pY) * j;
        if (sX > eX && sX < eX + dX && sY > eY && sY < eY + dY) {
          appuye = ! appuye;
          dessineBtn(eX, eY, String(touche[i + nbC * j]), appuye);
          dessinTouche(i, j);
          while (myTouch.dataAvailable()) // attend que l'appui sur l'écran cesse
            myTouch.read();
          if (retourHaut) {
            appuye = false;
            dessineBtn(eX, eY, String(touche[i + nbC * j]), false);
            dessinTouche(i, j);
          }
          if (surAppui != NULL)
            surAppui(appuye);
          if (i + nbC * j != 20)
            ptrEdt->addChar(String(touche[i + nbC * j]));
          switch (i + nbC * j) {
            case 20: // passage de majuscule à minuscule puis chiffres
              MajToMinToNb++;
              if (MajToMinToNb == 3)
                MajToMinToNb = 0;
              choixSaisie(MajToMinToNb);
              affiche();
              break;
            case 28: // touche annuler dernier caractère entré
              ptrEdt->debTexte = ptrEdt->debTexte.substring(0, ptrEdt->debTexte.length() - 1);
              ptrEdt->affiche();
              break;
            case 29: // touche OK
              visible = false;
              ptrEdt->clEditeur = VGA_WHITE;
              myGLCD.fillScr(clFond);
              activiteComp(true); // réactive tous les composants
              installeComp(ongletActif); // ré-affiche la page en cours
              ptrEdt->majValeur(ptrEdt->texte); // mise à jour de la valeur dans l'éditeur attaché
              if (ptrEdt->surQuitter != NULL)
                ptrEdt->surQuitter(MajToMinToNb); // fonction externe
              ptrEdt->maj = false;
              break;
          }
          break;
        }
      }
    return true;
  }
  else
    return false;
}

/*-----------------------------------------------------*/
/*                          ONGLETS                          */
/*-----------------------------------------------------*/

int Onglet::init(byte no, int p, byte noA, byte nbOng, String ti, int cl) {
  clOnglet = cl;
  rang = no;
  pos = p;
  titre = ti;
  larg = titre.length() * fontDimX;
  if (no == noA)
    actif = true;
  else
    actif = false;
  if (rang == nbOng - 1) {
    if (actif)
      myGLCD.drawImg(pos + larg + 8, 0, 8, 24, faAct);
    else
      myGLCD.drawBitmap(pos + larg + 8, 0, 8, 24, fa);
  }
  if (rang == 0) {
    if (actif)
      myGLCD.drawImg(pos, 0, 8, 24, oaAct);
    else
      myGLCD.drawBitmap(pos, 0, 8, 24, oa);
  }
  else {
    if (actif) {
      myGLCD.drawImg(pos, 0, 8, 24, fi_oaAct);
    }
    else {
      if (rang < noA || rang > noA + 1)
        myGLCD.drawBitmap(pos, 0, 8, 24, fi_oi);
      else
        myGLCD.drawImg(pos, 0, 8, 24, fa_oiAct);
    }
  }
  myGLCD.setColor(VGA_BLACK);
  myGLCD.drawLine(pos + 8, 0, pos + larg + 8, 0);
  if (actif) {
    myGLCD.setColor(clOnglet);
    myGLCD.fillRect(pos + 8, 0, pos + larg + 8, 23);
    myGLCD.setBackColor(clOnglet);
  }
  else {
    myGLCD.setColor(0xE71C); // couleur de fond d'un onglet inactif
    myGLCD.fillRect(pos + 8, 0, pos + larg + 8, 23);
    myGLCD.setColor(VGA_BLACK);
    if (rang < noA || rang > noA + 1)
      myGLCD.drawLine(pos, 23, pos + larg + 15, 23);
    else
      myGLCD.drawLine(pos + 8, 23, pos + larg + 15, 23);
    myGLCD.setBackColor(0xE71C);
  }
  myGLCD.setColor(VGA_BLACK);
  myGLCD.print(titre, pos + 8, 8);
  return pos + larg + 8;
}

/*-----------------------------------------------------*/
/*                             PAGES                             */
/*-----------------------------------------------------*/

Pages::Pages(String ti, int cl, byte p) {
  tag = 0x21;
  clPages = cl;
  titres = ti;
  for (int i = 0; i < 0xC0; i++) { // modifie les couleurs des bitmaps séparant les onglets pour les parties actives
    oaAct[i] = pgm_read_word(&oa[i]);
    if (oaAct[i] == 0xE71C)
      oaAct[i] = 0xF798;
    faAct[i] = pgm_read_word(&fa[i]);
    if (faAct[i] == 0xE71C)
      faAct[i] = 0xF798;
    fi_oaAct[i] = pgm_read_word(&fi_oa[i]);
    if (fi_oaAct[i] == 0xFFFF)
      fi_oaAct[i] = 0xF798;
    fa_oiAct[i] = pgm_read_word(&fa_oi[i]);
    if (fa_oiAct[i] == 0xFFFF)
      fa_oiAct[i] = 0xF798;
  }
  byte j = 0;
  int p1 = 0;
  int p2;
  for (int i = 0; i < titres.length(); i++) { // séparation des titres
    if (titres[i] == '&') {
      p2 = i;
      tiOng[j] = titres.substring(p1, p2);
      j++;
      p1 = p2 + 1;
    }
  }
  tiOng[j] =  titres.substring(p1);
  nbOnglets = j + 1;
  page = 0xFF; // visible sur toutes les pages
  noActif = p; // page active au lancement
  ongletActif = p;
}

void Pages::affiche() {
  int p1 = 0;
  for (byte i = 0; i < nbOnglets; i++) // initialisation des onglets
    p1 = onglet[i].init(i, p1, noActif, nbOnglets, tiOng[i], clPages);
  myGLCD.setColor(VGA_WHITE);
  myGLCD.fillRect(p1 + 8, 0, myGLCD.getDisplayXSize() - 1, 22); // efface l'espace à droite des titres de l'onglet
  myGLCD.setColor(VGA_BLACK);
  myGLCD.drawLine(p1 + 8, 23, myGLCD.getDisplayXSize() - 1, 23); // continue la ligne jusqu'au bord droit de l'écran
  myGLCD.setColor(clPages);
  myGLCD.fillRect(0, 24, myGLCD.getDisplayXSize() - 1, myGLCD.getDisplayYSize() - 1); // efface l'espace de travail de l'onglet
}

void Pages::changeOnglet(byte noA) { //
  noActif = noA;
  ongletActif = noA;
  int p1 = 0;
  affiche();
  installeComp(noA); // active les composants relatifs à l'onglet actif et désactive tous les autres
}

boolean Pages::select(int sX, int sY) { // détecte si la position d'appui sur l'écran appartient à un onglet
  boolean res = false;
  if (actif) {
    byte slc = 0xFE;
    for (byte i = 0; i < nbOnglets; i++)
      if (i != noActif && sX > onglet[i].pos && sX < onglet[i].pos + onglet[i].larg && sY > 0 && sY < 23)  {
        slc = i;
        changeOnglet(slc);
        if (surAppui != NULL)
          surAppui(slc);
        res = true;
        break;
      }
  }
  return res;
}

/*-----------------------------------------------------*/
/*                          CURSEUR                           */
/*-----------------------------------------------------*/

Curseur::Curseur(int a, int b, int c, byte po, byte p) {
  tag = 0x07;
  x = a;
  y = b;
  dX = c;
  pos = po;
  precPos = x + pos * dX / 100;
  curPos = precPos;
  cur = CurseurI;
  page = p;
}

void Curseur::affiche() {
  myGLCD.setBackColor(clFond);
  if (visible) {
    myGLCD.setColor(clFond);
    myGLCD.fillRect(precPos - 8, y - 4, precPos + 8, y + 12);
    myGLCD.setColor(VGA_SILVER);
    if (curPos == precPos)
      myGLCD.fillRect(x - 8, y, x + dX + 8, y + 7);
    else if (curPos > precPos)
      myGLCD.fillRect(precPos - 8, y, curPos + 8, y + 7);
    else
      myGLCD.fillRect(curPos - 8, y, precPos + 8, y + 7);
    myGLCD.setColor(VGA_BLACK);
    for (byte i = 0; i < 11; i++) {
      int p = x + i * dX / 10;
      myGLCD.drawLine(p, y + 2, p, y + 5);
    }
    myGLCD.drawTransparentBitmap(curPos - 8, y - 4, 16, 16, CurseurI);
    precPos = curPos; // mémorisation de la position précédente
  }
}

void Curseur::efface() {
  visible = false;
  myGLCD.setBackColor(clFond);
  myGLCD.setColor(clFond);
  myGLCD.fillRect(curPos - 8, y - 4, curPos + 8, y + 12); // efface le curseur
  myGLCD.fillRect(x - 8, y, x + dX + 8, y + 7); // efface la ligne
}

boolean Curseur::select(int sX, int sY) { // détecte si la position d'appui sur l'écran appartient à un onglet
  if (actif && visible && sX >= x && sX <= x + dX /*&& sX > curPos-8 && sX < curPos+8*/ && sY > y - 4 && sY < y + 12) {
    if (valide) {
      curPos = sX;
      pos = (curPos - x) * 100 / dX;
      if (pos < 0)
        pos = 0;
      if (pos > 100)
        pos = 100;
      affiche();
      if (surAppui != NULL)
        surAppui(pos);
      return true;
    }
    else
      return false;
  }
  else
    return false;
}

/*-----------------------------------------------------*/
/*                           INDEX                             */
/*-----------------------------------------------------*/

Index::Index(int a, int b, int c, byte po, char d, byte p) {
  tag = 0x07;
  x = a;
  y = b;
  dX = c; // Etendue de balayage du curseur
  pos = po;
	dir = d;
	int t;
	switch(dir) {
		case 'N':
			cur = C_Nord;
			dH = -3;
			dV = 0;
			t = x;
			break;		
		case 'E':
			cur = C_Est;
			dH = -6;
			dV = -3;
			t = y;
			break;
		case 'S':
			cur = C_Sud;
			dH = -3;
			dV = -6;
			t = x;
			break;
		case 'O':
			cur = C_Ouest;
			dH = 0;
			dV = -3;
			t = y;
			break;
	}
  curPos = t + pos * dX / 100;
  precPos = curPos;
  page = p;
}

void Index::affiche() {
  myGLCD.setBackColor(clFond);
  if (visible) {
		myGLCD.setColor(VGA_BLACK);
    myGLCD.setColor(clFond);
		switch (dir) {
			case 'N':
			case 'S':
				myGLCD.fillRect(precPos + dH, y + dV, precPos - dH, y + 7 + dV);
				myGLCD.drawTransparentBitmap(curPos + dH, y + dV, 7, 7, cur);
				break;
			case 'E':
			case 'O':
				myGLCD.fillRect(x + dH, precPos + dV, x + 6 + dH, precPos + 6 + dV);
				myGLCD.drawTransparentBitmap(x + dH, curPos + dV, 7, 7, cur);
				break;
		}
    precPos = curPos; // mémorisation de la position précédente
  }
}

void Index::efface() {
  visible = false;
  myGLCD.setBackColor(clFond);
  myGLCD.setColor(clFond);
	switch (dir) {
		case 'N':
		case 'S':
			myGLCD.fillRect(curPos + dH, y, curPos - dH, y + 7 + dV); // efface le curseur
			break;
		case 'E':
		case 'O':
			myGLCD.fillRect(x + dH, precPos + dV, x + 7 + dH, precPos + 7 + dV);
			break;
	}
}

boolean Index::select(int sX, int sY) { // détecte si la position d'appui sur l'écran appartient à un onglet
	boolean detect = false;
	switch (dir) {
		case 'N':
			detect = sX >= x && sX <= x + dX  && sY > y && sY < y + 7;
			break;
		case 'S':
			detect = sX >= x && sX <= x + dX  && sY > y + dV && sY < y + 7 + dV;
			break;
		case 'E':
			detect = sX >= x + dH && sX <= x && sY > y && sY < y + dX;
			break;
		case 'O':
			detect = sX >= x && sX <= x + 7  && sY > y && sY < y + dX;
			break;
	}
  if (actif && visible && detect) {
    if (valide) {
			switch (dir) {
				case 'N':
				case 'S':
					curPos = sX;
					pos = (curPos - x) * 100 / dX;
					break;
				case 'E':
				case 'O':
					curPos = sY;
					pos = (curPos - y) * 100 / dX;
					break;
			}
      if (pos < 0)
        pos = 0;
      if (pos > 100)
        pos = 100;
      affiche();
      if (surAppui != NULL)
        surAppui(pos);
      return true;
    }
    else
      return false;
  }
  else
    return false;
}


/*-----------------------------------------------------*/
/*                            FORME                            */
/*-----------------------------------------------------*/

Forme::Forme(int a, int b, int c, int d, int cl, char tf, byte np, byte p) {
  // tf = type de forme : 'L' : ligne ; 'C' : cercle ; 'R' : rectangle ; 'T' : triangle ; 'P' : polygone
  tag = 0x30;
  x = a;
  y = b;
  dX = c;
  dY = d;
  clTrait = cl;
  typeForme = tf;
  nbPts = np;
  ptP = new Pt[np]; // création du tableau de points nécessaires au dessin de la forme
  page = p;
}

void Forme::affiche() {
  myGLCD.setBackColor(clFond);
  if (visible)
    myGLCD.setColor(clTrait);
  else
    myGLCD.setColor(clFond);
  switch (typeForme) {
    case 'L' : // on n'utilise pas le tableau de points, mais les données x, y, dX et dY
      myGLCD.drawLine(x, y, x + dX, y + dY);
      break;
    case 'C' : // on n'utilise pas le tableau de points, mais les données x, y, dX et dY
      myGLCD.drawCircle(x, y, dX);
      break;
    case 'R' :
      myGLCD.drawRoundRect(x, y, x + dX, y + dY);
      break;
    case 'T' : // le triangle est un polygone
    case 'P' :
      for (byte i = 0; i < nbPts - 1; i++)
        myGLCD.drawLine(ptP[i].x, ptP[i].y, ptP[i + 1].x, ptP[i + 1].y);
      myGLCD.drawLine(ptP[nbPts - 1].x, ptP[nbPts - 1].y, ptP[0].x, ptP[0].y); // on ferme le polygone
      break;
  }
}

void Forme::efface() {
  visible = false;
  affiche();
}

/*-----------------------------------------------------*/
/*                            IMAGE                            */
/*-----------------------------------------------------*/

Image::Image(int a, int b, int c, int d, bitmapdatatype im, byte p) {
  tag = 0x06;
  x = a;
  y = b;
  dX = c;
  dY = d;
  img = im;
  page = p;
}

void Image::affiche() {
  if (visible)
    myGLCD.drawTransparentBitmap(x, y, dX, dY, img);
}

void Image::efface() {
  visible = false;
  myGLCD.setColor(clFond);
  myGLCD.fillRect(x, y, x + dX, y + dY);
}

boolean Image::select(int sX, int sY) {
  if (actif && visible && sX > x && sX < x + dX && sY > y && sY < y + dY) {
    if (valide) {
      myGLCD.setColor(VGA_BLUE);
      myGLCD.drawRect(x - 1, y - 1, x + dX, y + dY);
      myGLCD.drawRect(x - 2, y - 2, x + dX + 1, y + dY + 1);
      while (myTouch.dataAvailable()) // attend que l'appui sur l'écran cesse
        myTouch.read();
      myGLCD.setColor(clFond);
      myGLCD.drawRect(x - 1, y - 1, x + dX, y + dY);
      myGLCD.drawRect(x - 2, y - 2, x + dX + 1, y + dY + 1);
      if (surAppui != NULL)
        surAppui(appuye);
      return true;
    }
    else
      return false;
  }
  else
    return false;
}

/*-----------------------------------------------------*/
/*                          MESSAGE                          */
/*-----------------------------------------------------*/

Message::Message() {
  tag = 0x12;
  actif = false;
  dY = 24;
  y = (myGLCD.getDisplayYSize() - dY) / 2; // centré verticalement
  page = 0xFF; // affichage sur n'importe quelle page
}

void Message::affiche(String txt, char inf, byte act) {
  switch (inf) {
    case 'I': // information
      icone = Info;
      break;
    case 'A': // Attention !
      icone = Warning;
      break;
    case 'E': // erreur
      icone = Stop;
      break;
  }
  titre = txt;
  action = act;
  dX = txt.length() * fontDimX + 24;
  x = (myGLCD.getDisplayXSize() - dX) / 2;
  activiteComp(false); // désactive tous les composants ...
  actif = true; // ... sauf lui-même
  visible = true;
  Bouton::affiche();
}

boolean Message::select(int sX, int sY) {
  if (actif && visible && sX > x && sX < x + dX && sY > y && sY < y + dY) {
    if (valide) {
      appuye = ! appuye;
      Bouton::affiche(appuye);
      while (myTouch.dataAvailable()) // attend que l'appui sur l'écran cesse
        myTouch.read();
      appuye = false;
      activiteComp(true); // réactive tous les composants
      installeComp(ongletActif); // ré-affiche la page en cours
      actif = false;
      if (surAppui != NULL)
        surAppui(appuye);
      Bouton::efface();
      return true;
    }
    else
      return false;
  }
  else
    return false;
}

/*-----------------------------------------------------*/
/*                            VEILLE                           */
/*-----------------------------------------------------*/

Veille::Veille(long att) {
  attente = att;
  dureeInactif = millis();
  enVeille = false;
  tag = 0xF0;
  page = 0xFF; // affichage sur n'importe quelle page
}

boolean Veille::select(int sX, int sY) {
  if (actif) {
    if (sX == 0 && sY == 0) { // déclenché par l'inactivité
      if (millis() >= attente + dureeInactif) {
        myGLCD.fillScr(VGA_BLACK);
        enVeille = true;
        dureeInactif = 0xFFFFFFFF - attente;
      }
    } else {	// déclenché par un appui sur l'écran
      dureeInactif = millis();
      if (enVeille) {
        activiteComp(true); // réactive tous les composants
        installeComp(ongletActif); // ré-affiche la page en cours
        enVeille = false;
        if (surAppui != NULL)
          surAppui(true);
      }
    }
    return true;
  }
  else
    return false;
}
