#ifndef BaseComp_h
#define BaseComp_h

#include <Arduino.h>
#include <UTFT.h>
#include <UTouch.h>

#define BMP_TRANSPARENT	0x8010 // Purple

#define FOND_ECRAN   0xF798
#define BTN_TRAIT    0xA514
#define BTN_RELACHE  0xF79C
#define BTN_APPUYE   0xC618

#define ALIGN_GAUCHE  6
#define HAUTEUR_CLAVIER  103
#define MAJUSCULES      "AZERTYUIOPQSDFGHJKLM  WXCVBN"
#define MINUSCULES      "azertyuiopqsdfghjklm  wxcvbn"
#define CHIFFRES_PONCT  "0123456789+-*/=,:;._  '\"!?()"

class UTFT_M : public UTFT { // extension de la classe UTFT pour y ajouter la méthode "drawImg"
  public:
    UTFT_M(byte model, int RS, int WR, int CS, int RST, int SER = 0);
    void drawImg(int x, int y, int sx, int sy, bitmapdatatype data); // dessine une image à partir d'un fichier en SRAM et non en Flash
    void drawTransparentBitmap(int x, int y, int sx, int sy, bitmapdatatype data);
    void drawTransparentBitmap(int x, int y, int sx, int sy, bitmapdatatype data, int deg, int rox, int roy);
    void drawArc(int x, int y, int r, float a1, float a2);
};

extern UTFT_M myGLCD;
extern UTouch myTouch;

extern unsigned long dureeInactif;
extern int clFond;
extern int scrDimX;
extern int scrDimY;
extern byte fontDimX;
extern byte fontDimY;
extern void activiteComp(boolean act);

extern unsigned int oa[0xC0];
extern unsigned int fa[0xC0];
extern unsigned int fi_oa[0xC0];
extern unsigned int fa_oi[0xC0];
extern unsigned int fi_oi[0xC0];

extern unsigned int Espace[0x144];
extern unsigned int Efface[0x144];
extern unsigned int OK[0x144];
extern unsigned int Maj[0x1C0];
extern unsigned int Min[0x1C0];
extern unsigned int NbPonct[0x1C0];
extern unsigned int Caret[0x023];
extern unsigned int CheckBox0[0x0A9];
extern unsigned int CheckBox1[0x0A9];
extern unsigned int BtnRad0[0x0A9];
extern unsigned int BtnRad1[0x0A9];
extern unsigned int BtnAnnule[0x100];
extern unsigned int BtnOK[0x100];
extern unsigned int Courbes[0x100];
extern unsigned int Enrg[0x100];
extern unsigned int FlecheB[0x20D];
extern unsigned int Horloge[0x100];
extern unsigned int Drapeau[0x0A9];
extern unsigned int Info[0x100];
extern unsigned int Warning[0x100];
extern unsigned int Stop[0x100];
extern unsigned int CurseurI[0x100];
extern unsigned int C_Nord[0x31];
extern unsigned int C_Est[0x31];
extern unsigned int C_Sud[0x31];
extern unsigned int C_Ouest[0x31];
extern unsigned int SgnPlus[0x100];
extern unsigned int SgnMoins[0x100];

int bmp24To16(long bmp24);

void defEcran(int HouV, int clF = FOND_ECRAN);

class CompBase; // déclaration forward de CompBase

extern int nbComp;
extern byte ongletActif;
extern boolean creation;

struct PtrComp { // liste chaînée pour l'enregistrement des composants créés
  CompBase * ptrB; // élément CompBase
  PtrComp * suiv; // pointeur sur le prochain élément de la liste
  PtrComp * ajoutComp(CompBase * pt) { // procédure d'ajout pour les composants CompBase
    PtrComp * nouvComp = new PtrComp;
    nouvComp->ptrB = pt;
    suiv = nouvComp;
    return nouvComp;
  }
};

extern PtrComp * compDeb;
extern PtrComp * compCour;

/*-----------------------------------------------------
                     Tableau des tag
	CompBase		0x00
	Bouton			0x01
	Editeur			0x02
	BoiteListe		0x10
	Clavier			0x11
	Label				0x03
	Cadre			0x08
	BoiteTitre		0x31
	Marque			0x04
	BoutonChoix	0x05
	Pages			0x21
	Curseur			0x07
	Index			0x09
	Forme			0x30
	Image			0x06
	Message		0x12
	InfoDH			0xF0
	Veille			0xF0
  -----------------------------------------------------*/

class CompBase {
  protected:
  public:
    byte tag;
    int x, y, dX, dY;
    CompBase() { // constructeur
      tag = 0x00;
      compCour = compCour->ajoutComp(this); // inscription dans la liste chaînée de composants
      if (nbComp == 0)
        compDeb = compCour; // fixe un pointeur sur le début de la liste
      nbComp++; // incrémente l'indice d'inscription dans le tableau des composants
      x = 0; y = 0; dX = 0; dY = 0;
      surAppui = NULL;
      actif = true;
      valide = true;
      visible = true;
      appuye = false;
      titre = "";
      clTitre = VGA_BLACK;
      groupe = 0;
      page = 0;
    }
    virtual ~CompBase() { // destructeur
      PtrComp * compAct = compDeb;
      if (compAct->ptrB == this) {
        PtrComp * ptrSuppr = compDeb;
        compDeb = compDeb->suiv;
        nbComp--;
        delete ptrSuppr;
      }
      else {
        for (byte i = 0; i < nbComp; i++) {
          if (compAct->suiv->ptrB == this) {
            PtrComp * ptrSuppr = compAct->suiv;
            compAct->suiv = compAct->suiv->suiv;
            nbComp--;
            delete ptrSuppr;
            break;
          }
          else
            compAct = compAct->suiv;
        }
      }
    }
    boolean actif; // le composant appartient à l'onglet actif
    boolean valide; // permet de contrôler de l'extérieur si un composant peut être sélectionné
    boolean visible;
    boolean appuye;
    String titre;
    int clTitre;
    byte groupe;
    byte page; // dans le cas d'utilisation avec des onglets, "page" doit être égal à l'index de l'onglet sur lequel est posé le composant
    virtual void init();
		void (* surAppui)(boolean); // pointeur sur une fonction d'action en cas d'appui sur le composant
    virtual boolean select(int, int);
    virtual void affiche(boolean);
    virtual void affiche();
    virtual void efface();
};

class Bouton : public CompBase {
  protected:
    void dessineBtn(int, int, String, boolean);
  public:
    Bouton() {};
    Bouton(int, int, int, int, String, bitmapdatatype, boolean, byte = 0);
    ~Bouton() {};
    boolean exclusif;
    bitmapdatatype icone;
    boolean retourHaut;
    virtual void affiche();
    void affiche(boolean); // dessine le composant
    void efface();
    virtual boolean select(int, int);
};

class Editeur : public CompBase {
  private:
  protected:
    int posCaret;
    int posAlign;
    int placeCaret(int);
  public:
    Editeur(int, int, int, int, String, byte, char, boolean, byte = 0);
    int clEditeur;
    int clTexte;
    boolean maj;
    boolean lectureSeule;
    String debTexte;
    String finTexte;
    String texte;
    char align;
    byte typeSaisie; // 0 = Majuscules ; 1 = minuscules ; 2 = chiffres
    void addChar(String);
    virtual void majValeur(String);
    virtual void affiche();
    virtual void affiche(String);
    void efface();
    void (* surQuitter)(byte); // pointeur sur une fonction d'action quand on appuie sur la touche OK du clavier
    virtual boolean select(int, int);
};

class BoiteListe : public Editeur {
  private:
    int hB, yH, yB; // hauteur, bord haut et bas de la boîte ouverte
    byte hL; // hauteur d'une ligne d'item
    boolean listeOuverte;
    bitmapdatatype icone;
    int hauteurBoite();
  public:
    BoiteListe(int, int, int, String, char, boolean, byte = 0);
    String choix;
    String items;
    byte nbItems;
    byte index;
    byte assigneItems(String);
    byte ajoutStr(String, byte);
    byte supprStr(byte);
    byte supprStr(String);
    virtual void majValeur(String);
    void affiche();
    void affiche(String);
    virtual boolean select(int, int);
};

class Clavier : public Bouton {
  private:
    byte nbC; // nombre de colonnes
    byte nbL; // nombre de lignes
    byte pX; // espacement entre les touches en X
    byte pY; // espacement entre les touches en Y
    byte maG; // marge gauche
    byte maB; // marge des bmp dans les boutons
    int hC; // haut du clavier
    String touche;
    byte MajToMinToNb;
    unsigned int *ptrBmp[10];
    void choixSaisie(byte);
  public:
    Editeur * ptrEdt;
    Clavier();
    void montre(int, byte);
    void affiche();
    void dessinTouche(int, int);
    boolean select(int, int);
};

extern Clavier * clavier;

class Label : public CompBase {
  public:
    Label(int, int, String, int, byte = 0);
    void affiche();
    void affiche(String);
    void efface();
};

class Cadre : public CompBase {
  public:
    Cadre(int, int, int, int, String, int, int, byte = 0);
    int clCadre;
    void affiche();
    void affiche(String);
    void efface();
};

class BoiteTitre : public CompBase {
  public:
    BoiteTitre(int, int, int, int, String, int, int, byte = 0);
    int clLabel;
    int clCadre;
    void affiche();
    void affiche(String);
    void efface();
};

class Marque : public CompBase {
  public:
    Marque(int, int, String, int, byte = 0);
    void (* surAppui)(boolean); // pointeur sur une fonction d'action en cas d'appui sur le composant
    int clMarque;
    boolean coche;
    void affiche();
    void efface();
    boolean select(int, int);
};

class BoutonChoix : public CompBase {
  public:
    BoutonChoix(int, int, String, int, byte, byte, byte = 0);
    void (* surAppui)(byte, String); // pointeur sur une fonction d'action en cas d'appui sur le composant
    int clChoix;
    byte index;
    boolean coche;
    void affiche(boolean);
    void affiche();
    void efface();
    boolean select(int, int);
};

class Onglet {
  private:
    int clOnglet;
    byte rang;
    String titre;
    boolean actif;
  public:
    int pos;
    int larg;
    int init(byte, int, byte, byte, String, int);
};

class Pages : public CompBase {
  private:
  public:
    Pages(String, int, byte = 0);
    void (* surAppui)(byte); // pointeur sur une fonction d'action en cas d'appui sur le composant
    String titres;
    byte nbOnglets;
    byte noActif;
    int clPages;
    Onglet onglet[4]; // tableau de 4 onglets possibles
    String tiOng[4]; // tableau de 4 titres d'onglet
    void affiche();
    void changeOnglet(byte);
    boolean select(int, int);
};

class Curseur : public CompBase {
  private:
    int precPos;
    bitmapdatatype cur;
  public:
    Curseur(int, int, int, byte, byte = 0);
    ~Curseur() {}
    int pos, curPos;
    void affiche();
    void efface();
    boolean select(int, int);
};

class Index : public CompBase {
  private:
    int precPos;
		int8_t dH, dV;
    bitmapdatatype cur;
  public:
    Index(int, int, int, byte, char, byte = 0);
    ~Index() {}
    byte pos, curPos;
		char dir;
    void affiche();
    void efface();
    boolean select(int, int);
};

class Forme : public CompBase {
  private:
    byte typeForme; // type de la forme
    byte nbPts;
    struct Pt {
      int x, y;
    };
  public :
    Forme() {};
    Forme(int, int, int, int, int, char, byte, byte = 0);
    ~Forme() {
      delete ptP;
    }
    int clTrait;
    Pt * ptP;
    void affiche();
    void efface();
};

class Image : public CompBase {
  private:
  public :
    Image() {};
    Image(int, int, int, int, bitmapdatatype, byte = 0);
    ~Image() {}
    bitmapdatatype img;
    void affiche();
    void efface();
    boolean select(int, int);
};

class Message : public Bouton {
  private:
    byte action; // index de l'action à prendre
  public :
    Message();
    ~Message() {}
    void affiche(String, char, byte);
    boolean select(int, int);
};

class Veille : public CompBase {
  public :
    Veille();
    Veille(long);
    ~Veille() {}
    unsigned long attente;
    boolean enVeille;
    boolean select(int, int);
};

void installeComp(int);
void scruteEvent();

String premStr(String, char, byte *);
byte nbChar(String, char);

#endif
