#include <Arduino.h>
#include <SdFat.h>
#include <CarteSD.h>
#include <DS3231.h>


DS3231 rtc(SDA, SCL);
boolean rtcOK = false;
Time dT;

const byte chipSelect = 53;

SdFat SD;
SdFile file;

/*-----------------------------------------------------*/
/*                           CARTESD                          */
/*-----------------------------------------------------*/

boolean CarteSD::init() {
	if(! rtcOK) {
		rtc.begin(); // Initialisation de l'horloge temps rel DS3231
		rtcOK = true;
	}
	if (SD.begin(chipSelect, SPI_HALF_SPEED))
		carteOK = true;
	else
		carteOK = false;
	return carteOK;
}

boolean CarteSD::fichierExiste(String nomFch) {
	if (carteOK) {
		char str[16];
		nomFch.toCharArray(str, 16);
		return SD.exists(str);
	}
	else
		return false;
}

boolean CarteSD::ouvreFichier(String nomFch, char mode) {
	if (carteOK) {
		boolean oK = false;
		int lg = nomFch.length()+2;
		char str[lg];
		nomFch.toCharArray(str, lg);
		switch(mode) {
		case 'C': // Cration pour criture avec effacement des donnes prcdentes
			if (file.open(str, O_RDWR | O_CREAT))
				oK = true;
			break;
		case 'E': // Ecriture en mode ajout
			if (file.open(str, O_RDWR | O_CREAT | O_AT_END))
				oK = true;
			break;
		case 'L': // Lecture
			if (file.open(str, O_READ))
				oK = true;
			break;
		}
		return oK;
	}
	else
		return false;
}

void CarteSD::ecritLigne(char stc) {
	if (carteOK) {
		file.println(stc);
	}
}

void CarteSD::ecritLigne(String st) {
	if (carteOK) {
		int lg = st.length()+2;
		char buf[lg];
		st.toCharArray(buf, lg);
		file.println(buf);
	}
}

void CarteSD::ecritLigne(byte b) {
	if (carteOK) {
		char str[8];
		sprintf(str, "%d", b);  
		file.println(str);
	}
}

void CarteSD::ecritLigne(int i) {
	if (carteOK) {
		char str[16];
		sprintf(str, "%d", i);  
		file.println(str);
	}
}

void CarteSD::ecritLigne(long l) {
	if (carteOK) {
		char str[16];
		sprintf(str, "%ld", l);
		file.println(str);
	}
}

void CarteSD::ecritLigne(float f) {
	if (carteOK) {
		char str[16];
		dtostre(f, str, 8, 3);
		file.println(str);
	}
}

void CarteSD::ecritLigne(double d) {
	if (carteOK) {
		char str[16];
		dtostre(d, str, 15, 3);
		file.println(str);
	}
}

String CarteSD::ecrit_Tc_Data(float dd[], byte nb) {
	String sTc = "";
	if (carteOK) {
		char stx[12+16*nb]; // stockage de la chane complte
		char st[16]; // stockage temporaire de chaque valeur
		sprintf(stx, "%lu", millis());
		sTc = stx;
		for (int i = 0; i < nb; i++) {
			dtostre(dd[i], st, 8, 3);
			strcat(stx, ";");
			strcat(stx, st);
		}  
		file.println(stx);
	}
	return sTc;
}

String CarteSD::ecrit_DT_Data(float dd[], byte nb) {
	dT = rtc.getTime();
	delay(2); // un dlai est ncessaire pour que rtc inscrive les donnes dans dT. Apparemment, 1 ms suffit
	char stx[24+16*nb]; // stockage de la chane complte
	sprintf(stx, "%02u.%02u.%04u;%02u:%02u:%02u", dT.date, dT.mon, dT.year, dT.hour, dT.min, dT.sec);
	String sDT = stx;
	if (carteOK) {
		char st[16]; // stockage temporaire de chaque valeur
		for (int i = 0; i < nb; i++) {
			dtostre(dd[i], st, 8, 3);
			strcat(stx, ";");
			strcat(stx, st);
		}  
		file.println(stx);
	}
	return sDT;
}

void CarteSD::litLigne(char * c) {
}

boolean CarteSD::donneesValides() {
	return file.available() > 0;
}

void CarteSD::litLigne(String * s) {
	* s = "";
	byte b;
	byte n = 0;
	while ( true) {
		b = file.read();
		n++;
		if (b != 13) // caractre "retour chariot"
			* s += char(b);
		else {
			file.read(); // lit le caractre " la ligne"
			break;
		}
	}
}

void CarteSD::litLigne(float * f) {
	char str[16];
	file.read(str, 16);
  * f = atof(str);
}

void CarteSD::litLigne(double * dd) {
	char str[16];
	file.read(str, 16);
  * dd = atof(str);
}

void CarteSD::fermeFichier() {
	file.close();
}

void CarteSD::supprFichier(char * f) {
	SD.remove(f);
}

String CarteSD::listeFchExt(String ext) {
	String lst = "";
  if (SD.begin(53, SPI_FULL_SPEED)) {
		char tmp[13];
		String fch = "";
		while (file.openNext(SD.vwd(), O_READ)) {
			file.getFilename(tmp);
			fch = tmp;
			if ((! file.isDir()) && fch.substring(fch.indexOf('.')).equalsIgnoreCase(ext))
				lst += '&'+fch.substring(0, fch.indexOf('.'));
			file.close();
		}
	}
  return lst;
}

/*-----------------------------------------------------*/
/*                           INFODH                          */
/*-----------------------------------------------------*/

InfoDH::InfoDH(int a, int b, boolean ls, byte p) : Editeur(a, b, 170, 24, "Date et Heure", 2, 'C', ls, p) {
	if(! rtcOK) {
		rtc.begin();
		rtcOK = true;
	}
	majDH = false;
	tempsUNIX = 0;
	tag = 0xF0;
}

void InfoDH::majValeur(String t) { // Mise  jour (date et heure de l'horloge temps rel
  rtc.setDate(t.substring(0, 2).toInt(), t.substring(3, 5).toInt(), t.substring(6, 10).toInt());
  rtc.setTime(t.substring(11, 13).toInt(), t.substring(14, 16).toInt(), t.substring(17).toInt());
	majDH = false;
}

boolean InfoDH::select(int sX, int sY) {
	if (actif && visible && sX > x && sX < x+dX && sY > y && sY < y+dY) {
		if ( ! lectureSeule) { // mode dition de texte
			if ( ! clavier->visible) { // on rend le clavier visible  pour dition de texte
				majDH = true;
				maj = true;
				clEditeur = VGA_AQUA;
				affiche("");
				activiteComp(false); // dsactive tous les composants ...
				actif = true; // ... sauf lui-mme
				clavier->actif = true;
				clavier->ptrEdt = this; // on envoie au clavier un pointeur sur l'diteur qui demande du texte
				if (y+dY+HAUTEUR_CLAVIER+5 > myGLCD.getDisplayYSize()) // pour ne pas masquer l'diteur avec le clavier, on place ce dernier l o l'diteur n'est pas
					clavier->montre(y-(HAUTEUR_CLAVIER+5), typeSaisie);
				else		
					clavier->montre(y+dY+5, typeSaisie);
			}
			posCaret = placeCaret(sX-(x+posAlign)+fontDimX/2); // mise en place du caret
		}
		if (surAppui != NULL)
			surAppui(lectureSeule); // fonction externe
		return true;
	}
	else if (sX ==0 && sY == 0 && !maj) { // affichage de la date et de l'heure
		dT = rtc.getTime();
//		delay(1); // un dlai est ncessaire pour que rtc inscrive les donnes dans dT. Apparemment, 1 ms suffit
		unsigned long tCour = rtc.getUnixTime(dT);
		if(tCour > tempsUNIX) {
			tempsUNIX = tCour;
			char stx[20]; // stockage de la chane complte
			sprintf(stx, "%02u.%02u.%04u %02u:%02u:%02u", dT.date, dT.mon, dT.year, dT.hour, dT.min, dT.sec);
			affiche(stx);
      if (surDelai != NULL)
        surDelai(tempsUNIX);
		}
		return true;
	}
	else 
		return false;
}