#include <Arduino.h>
#include <BaseComp.h>
#include <Courbes.h>

String floatToStr(float f) {
  byte nbD;
  char buf[6];
  if (abs(f) > 99.95)
    return String(int(f));
  else {
    if (abs(f) > 9.995)
	    nbD = 1;
	  else
      nbD = 2;
    dtostrf(f, 4, nbD, buf);
	return buf;
  }		
}
String tempsEcoule(long t) {
  char buf[14];
  byte s = t%60;
  t /= 60;
  byte m = t%60;
  t /= 60;
  byte h = t%24;
  int j = (t/24)%365;
  sprintf(buf, "%03d  %02d:%02d:%02d", j, h, m, s);
  return buf;
}

/*-----------------------------------------------------*/
/*                       AFFCOURBES                       */
/*-----------------------------------------------------*/

AffCourbes::AffCourbes(byte dE, String iC, float mi, float mx, int np) : edtMinY(64, 0, 55, 24, floatToStr(mi), 2, 'D', false, 0xFD), edtMaxY(175, 0, 55, 24, floatToStr(mx), 2, 'D', false, 0xFD), fermer(0, 0, 80, 24, "Fermer", BtnAnnule, true, 0xFD) {
  debut = true;
  visible = false;
  actif = true;
  page = 0xFD; // visible sur toutes les pages
  x = 23;
	dTop = dE; // dTop est le dcalage par rapport au bord haut de l'cran
  y = 12+dTop;
  dH = 3;
  axeH = new int[dH];
  valH = new float[dH+1];
  infosCrbs = iC;
  lT = infosCrbs.length(); // nombre de caractres de la chane de texte
  byte j = 0;
  for (byte i = 0; i < lT; i++)
    if ( infosCrbs[i] == '&') { // le nombre de courbes  afficher est gal au nombre de titres de courbe
      j++;
    }
  nbCrbs = j-1;
  nbPts = np; // nombre de points sur l'axe horizontal
  noPt = 0;
  nomCrb = new String[nbCrbs];
  clCrb = new int[nbCrbs];
  ptCour = new float[nbCrbs];
  valC = new float*[nbCrbs];
  for (byte i = 0; i < nbCrbs; i++)
    valC[i] = new float[nbPts+1];
  pos = new byte[nbCrbs+2];
  j = 0;
  for (byte i = 0; i < lT; i++)
    if ( infosCrbs[i] == '&') {
      pos[j] = i; // Position de l'affichage des titres de courbe
      j++;
    }
  pos[j] = lT;
	pas = 1;
	miniY = 0;
	dT = 1;
	axeV = NULL;
	nbPtAff = nbPts;
	kA = 1;
	zA = 0;
}

void AffCourbes::init() { 
  if (myGLCD.orient == PORTRAIT) {
    dV = 3;
    dX = myGLCD.getDisplayXSize()-x-5;
    dY = myGLCD.getDisplayYSize()-y-80;
  }
  else {
    dV = 5;
    dX = myGLCD.getDisplayXSize()-x-5;
	dY = myGLCD.getDisplayYSize()-y-50;
  }
  axeV = new int[dV];
  valV = new float[dV+1];
  edtMaxY.y = y+dY+22;
  edtMinY.y = y+dY+22;
  pas = float(dX)/float(nbPts);
  fermer.x = myGLCD.getDisplayXSize()-85;
  fermer.y = myGLCD.getDisplayYSize()-28;
}

void AffCourbes::minmaxY(float yMin, float yMax) {
  miniY = yMin;
  maxiY = yMax;
  edtMinY.texte = floatToStr(yMin);
  edtMaxY.texte = floatToStr(yMax);
  if (actif && visible) {
    affiche();
    edtMinY.affiche(floatToStr(miniY));
    edtMaxY.affiche(floatToStr(maxiY));
  }
}

void AffCourbes::montre(float dt) {
  dT = dt/1000.0;
//  myGLCD.fillScr(clFond);
  visible = true;
  ongletAppel = ongletActif; // mmorise de quel onglet vient l'appel
  ongletActif = 0xFD; // onglet de l'afficheur de courbes
  installeComp(ongletActif); // Apparemment, cette instruction efface le texte des diteurs ?????
  edtMinY.affiche(floatToStr(miniY));
  edtMaxY.affiche(floatToStr(maxiY));
}

void AffCourbes::affiche() {
  if ( debut) { // pendant la phase d'initialisation
    debut = false;
    for(int i = 0; i < dV; i++) 
      axeV[i] = x+dX*(i+1)/(dV+1);
    for(int i = 0; i <= dH; i++)
      axeH[i] = y+dY*(i+1)/(dH+1);
  } 
  if (visible) {
    myGLCD.fillScr(clFond);
    myGLCD.setColor(0xFFF8);
    myGLCD.fillRect(x-1, y, x+dX+1, y+dY);
    myGLCD.setColor(VGA_BLACK);
    myGLCD.drawRect(x-1, y, x+dX+1, y+dY);
    myGLCD.setColor(VGA_BLUE);
    myGLCD.setBackColor(clFond);
	  myGLCD.print(infosCrbs.substring(0, pos[0]), CENTER, y-10); // affichage du titre gnral
		miniY = edtMinY.texte.toFloat();
		maxiY = edtMaxY.texte.toFloat();
    kA = dY/(maxiY-miniY);
    zA = y+kA*maxiY;
    affGrad('V');  
    myGLCD.print(F("Y min"), 19,y+dY+30);
    myGLCD.print(F("Y max"), 130,y+dY+30);
    majAff();
  }
}

void AffCourbes::majAff() {
  if (visible && actif) {
    if(nbPtAff == nbPts) { // si l'affichage en est au dernier point ...
      myGLCD.setColor(0xFFF8);
      myGLCD.fillRect(x, y+1, x+dX, y+dY-1); //   l'arrive d'un nouveau point, on efface toute la surface d'affichage des courbes
    }
    myGLCD.setColor(VGA_BLACK);
    myGLCD.drawLine(x, y, x+dX, y); // bord haut
    myGLCD.drawLine(x, y+dY, x+dX, y+dY); // bord bas
    myGLCD.setColor(VGA_SILVER);
    for(byte i = 0; i < dV; i++)
      myGLCD.drawLine(axeV[i], y+1, axeV[i], y+dY-1); //  graduations verticales
    for(byte i = 0; i < dH; i++)
      myGLCD.drawLine(x, axeH[i], x+dX, axeH[i]); // graduations horizontales
  }
  if (noPt <= nbPts) {
    nbPtAff = noPt;
    for (byte j = 0; j < nbCrbs; j++) // prise en compte des nouveaux points
      valC[j][noPt] = ptCour[j];
  }
  else {
    nbPtAff = nbPts;
    for (int i = 0; i < nbPts; i++) { // dcalage des valeurs dans le registre
      for (byte j = 0; j < nbCrbs; j++)
        valC[j][i] = valC[j][i+1];
    }
    for (byte j = 0; j < nbCrbs; j++) // prise en compte des nouveaux points
      valC[j][nbPts] = ptCour[j];
  } 
  noPt++;
  if (visible && actif) {
    affGrad('H');
    for (byte j = 0; j < nbCrbs; j++) {
      myGLCD.setColor(clCrb[j]);
      for (int i = 0; i < nbPtAff ; i++) {
	    int valN = zA-kA*valC[j][i+1];
        if (valN < y)
          valN = y;
        else
          if (valN >= y+dY)
            valN = y+dY;
        int valNp = zA-kA*valC[j][i];
        if (valNp < y)
          valNp = y;
        else
          if (valNp >= y+dY)
            valNp = y+dY;
        myGLCD.drawLine(i*pas+x, /*zA-kA*valC[j][i]*/valNp, (i+1)*pas+x, valN);
      }
    }
  }
}

void AffCourbes::efface() {
}

boolean AffCourbes::select(int sX, int sY) {
  if (actif && visible && sX > fermer.x && sX < fermer.x+fermer.dX && sY > fermer.y && sY < fermer.y+fermer.dY) { // agit sur le bouton "fermer"
		if (surAppui != NULL)
			surAppui(appuye);
		else
			ongletActif = ongletAppel;
    visible = false;
    myGLCD.fillScr(clFond);
    activiteComp(true); // ractive tous les composants
    installeComp(ongletActif); // r-affiche la page en cours
  }
}

void AffCourbes::affGrad(char axe) {
  String str;
  myGLCD.setBackColor(clFond);
  switch(axe) {
  case 'H':
    str = tempsEcoule(millis()/1000);
    myGLCD.setColor(VGA_BLACK);
    myGLCD.print(str, myGLCD.getDisplayXSize()-5-str.length()*fontDimX, y+dY+3); // affichage du temps coul
    myGLCD.setColor(VGA_BLUE);
    char unit;
    byte k;
    if (dT > 86400) {
      unit = 'j'; // jours
      k = 86400;
    }
    else
      if (dT > 3600) {
        unit = 'h'; // heures
        k = 3600;
      }
      else
        if (dT > 60) {
          unit = 'm'; // minutes
          k = 60;
        }
        else {
          unit = 's'; // secondes
          k = 1;
        }
    str = "Nb points:"+String(nbPts)+"   dT:"+floatToStr(float(dT)/float(k))+" "+unit;
    myGLCD.print(str, myGLCD.getDisplayXSize()-5-str.length()*fontDimX, y+dY+12); // affichage des paramtres de l'axe X
    break;
  case 'V':
    myGLCD.setColor(clFond);
    myGLCD.fillRect(12, y, 10+fontDimY, y+dY+fontDimX/2);
    for(int i = 0; i <= dH; i++) { // affichage des valeurs des graduations de l'axe Y
      valV[i] = maxiY-(i+1)*(maxiY-miniY)/(dH+1);
      myGLCD.setColor(VGA_BLACK);
      str = floatToStr(valV[i]);	  
       myGLCD.print(str, 12, axeH[i]+str.length()*fontDimX/2, 270);
    }
    for (int i = 0; i <= nbCrbs; i++) {
      if (i < nbCrbs) { // affichage du nom des courbes
        myGLCD.setColor(clCrb[i]);
        myGLCD.print(infosCrbs.substring(pos[i]+1, pos[i+1]), 2, 1+(pos[nbCrbs]-pos[i])*fontDimX+dTop, 270); 
      }
      else { // affichage du nom des units de l'axe Y
        myGLCD.setColor(VGA_BLACK);
        myGLCD.print(infosCrbs.substring(pos[i]+1, pos[i+1]), 12, 2+(lT-pos[i]-1)*fontDimX+dTop, 270); 
      }	
    }
    break;
  }
}