#include <Arduino.h>
#include <SD.h>
#include <UtilsGPS.h>

#define SEC_JOUR 86400
int nbJ1J[] = {0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334};
int nbJM[] = {31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31};

void litReperes(File F, Reperes* rep, byte nb) {
  F = SD.open(F("Reperes.txt"), FILE_READ); // Ouvre le fichier de "repères"
  byte i = 0;
  while (F.available() && i < nb) {
    rep[i].nom = litLigne(F);
    rep[i].la = litLigne(F).toFloat();
    rep[i].lo = litLigne(F).toFloat();
    i++;
  }
  F.close(); // Ferme le fichier de repères
}

void introGPX(File F, String date) {
  F.println(F("<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?><gpx xmlns=\"http://www.topografix.com/GPX/1/1\" creator=\"GPS ChP\" version=\"1.1\" xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xsi:schemaLocation=\"http://www.topografix.com/GPX/1/1 http://www.topografix.com/GPX/1/1/gpx.xsd\"><metadata><time>"));
  F.print(date);
  F.print(F("</time></metadata><trk><name>"));
  F.print(F("Tracé"));
  F.println(F("</name><trkseg>"));
}

void trkGPX(File F, float lat, float lon, float alt, String date) {
  F.print(F("<trkpt lat=\""));
  F.print(String(lat, 6));
  F.print(F("\" lon=\""));
  F.print(String(lon, 6));
  F.print(F("\"><ele>"));
  F.print(String(alt, 1));
  F.print(F("</ele><time>"));
  F.print(date);
  F.println(F("</time></trkpt>"));
}

void finGPX(File F) {
  F.println(F("</trkseg></trk></gpx>"));
  F.close();
}

String litLigne(File F) {
  String s = "";
  char b;
  while ( true) {
    b = F.read();
    if (b != 0x0D) // caractère "retour chariot"
      s += b;
    else {
      F.read(); // lit le caractère "à la ligne"
      break;
    }
  }
  return s;
}

unsigned long nbSec1970(String DT) {
  int nbJours = (DT.substring(6, 10).toInt() - 1970) * 365 + (DT.substring(6, 10).toInt() - 1970) / 4;
  nbJours += nbJ1J[DT.substring(3, 5).toInt() - 1];
  if (DT.substring(6, 10).toInt() % 4 == 0 && DT.substring(3, 5).toInt() > 2) nbJours ++;
  nbJours += DT.substring(0, 2).toInt() - 1;
  return nbJours * SEC_JOUR + DT.substring(11, 13).toInt() * 3600 + DT.substring(14, 16).toInt() * 60 + DT.substring(17).toInt();
}

String dateHeure(unsigned long TU) {
  char dh[21];
  unsigned long TUprec = 0xFFFFFFFF;
  int a = 1970;
  while (TU <= TUprec) {
    TUprec = TU;
    if (a % 4 == 0)
      TU -= 366 * SEC_JOUR;
    else
      TU -= 365 * SEC_JOUR;
    a++;
  }
  a--;
  if (a % 4 == 0)
    nbJM[1] = 29;
  else
    nbJM[1] = 28;
  long Tr = TUprec;
  byte m;
  for (m = 0; m < 12; m++) {
    if (Tr - (nbJM[m]) * SEC_JOUR < 0)
      break;
    Tr -= nbJM[m] * SEC_JOUR;
  }
  byte j = Tr / SEC_JOUR;
  Tr = Tr % SEC_JOUR;
  byte h = Tr / 3600;
  Tr = Tr % 3600;
  byte mn = Tr / 60;
  byte s = Tr % 60;
  String formatAff = F("%02d/%02d/%04d %02d:%02d:%02d"); // format de date pour l'affichage (mis en mémoire flash)
  sprintf(dh, formatAff.c_str(), j + 1, m + 1, a, h, mn, s);
  return dh;
}

byte nbJoursMois(byte annee, byte mois) {
  return (mois==4 || mois==6 || mois==9 || mois==11) ? 30 : (mois==2 ? ((annee&3) ? 28 : 29) : 31);
}

void changerHeure(int *annee, byte *mois, byte *jour, byte *heure, int8_t decal) {
  if (decal > 0) {
    if (*heure < (24 - decal))
      (*heure) += decal;
    else {
      *heure = (*heure + decal) % 24;
      if (*jour < nbJoursMois(*annee, *mois))
        (*jour)++;
      else {
        *jour = 1;
        if (*mois < 12)
          (*mois)++;
        else {
          *mois = 1;
          (*annee)++;
        }
      }
    }
  }
  else if (decal < 0) {
    if (*heure + decal >= 0)
      (*heure) += decal;
    else {
      *heure = (*heure + decal + 24);
      if (*jour == 1) {
        (*mois)--;
        if (*mois == 0) {
          (*mois) = 12;
          (*annee)--;
        }
        *jour = nbJoursMois(*annee, *mois);
      } else {
        (*jour)--;
      }
    }
  }
}


