#include <Arduino.h>
#include <BaseComp.h>
#include <Oscilloscope.h>

#define BLOC_ACDC   98 // Position verticale du bloc AC-GND-DC
#define CORR_ZERO    8 // broche de sortie pour la correction de zro
#define ZERO_CAN   128
#define MUX_ACDC_0  A7
#define MUX_ACDC_1  A6
#define MUX_10kN_0  A5
#define MUX_10kN_1  A4
#define MUX_k125_0  A3
#define MUX_k125_1  A2

class ParamBT { // Paramtres de la Base de Temps
  public:
    String info; // dure par seconde affiche
    byte preDiv; // Pr diviseur du compteur
    byte compDiv; // valeur  laquelle le comptage reprend  zro
    char typeAff; // Type d'affichage  des courbes : 'N' = Normal ; 'P' = sous-chantillonn ; 'P' = Loupe
    byte sautPts; // Nombre de points sauts dans les mode 'P' et 'L'
    byte freqEch; // Frquence d'chantillonnage du CNA
};

ParamBT paramBT[] = { {"50$S", 0X01, 118, 'L', 4, 0xC2}, // 100 S (le $ reprsente le  dans la fonte)
  {"0.1mS", 0X01, 118, 'L', 2, 0xC2}, // 100 S
  {"0.2mS", 0X01, 118, 'N', 1, 0xC2}, // 200 S
  {"0.5mS", 0X02, 37, 'N', 1, 0xC3}, // 500 S
  {"1mS", 0X02, 74, 'N', 1, 0xC5}, // 1 mS
  {"2mS", 0X02, 148, 'N', 1, 0xC6}, // 2 mS
  {"5mS", 0X03, 93, 'N', 1, 0xC6}, // 5 mS
  {"10mS", 0X03, 185, 'N', 1, 0xC6}, // 10 mS
  {"20mS", 0X04, 185, 'N', 1, 0xC6}, // 20 mS
  {"50mS", 0X05, 231, 'N', 1, 0xC6}, // 50 mS
  {"0.1S", 0X06, 231, 'N', 1, 0xC6} // 100 mS
};

String calVert[] = {"10mV", "20mV", "50mV", "0.1V", "0.2V", "0.5V", "1V", "2V", "5V"};
byte sizeStrH = 11; // Taille du tableau paramBT
byte sizeStrV = 9; // Taille du tableau calVert
byte indxH; // Indice dans le tableau paramBT
byte indxV; // Indice dans le tableau  calVert
String txtH; // Texte slectionn pour calVert
String txtV; // Texte slectionn pour paramBT
boolean chgtOK = true; // Changement de sensibilit verticale (calVert)
boolean chgtOK_BT = true; // Changement de dure de la Base de Temps
byte modeTrig = 0; // Index du trigger
volatile int indR; // Index de l'chantillon analogique lue
volatile int res[NBPT]; // Valeur de l'chantillon analogique lue
int eff[NBPT_X + 1]; // Tableau des valeurs  effacer de l'affichage

ISR(TIMER2_COMPA_vect) {
  //  ADCSRA = B11000010; // Freq ADC = 16 MHz/4
  while ( ( ADCSRA & 0B00010000 ) == 0 ); // Attente de la fin de la conversion
  res[indR] = ADCH; // On ne lit que les 8 bits de poids forts
  //  res = ADCL | ADCH << 8;
  indR++;
  ADCSRA = ADCSRA | 0B01010000; // R-activation de CNA
}

/*-----------------------------------------------------*/
/*                              BTN                              */
/*-----------------------------------------------------*/

BTN::BTN(int a, int b, bitmapdatatype iH, bitmapdatatype iB, boolean rh, byte id, byte p) {
  tag = 0x01;
  x = a;
  y = b;
  dX = 16;
  dY = 16;
  exclusif = false;
  retourHaut = rh;
  appuye = false;
  indX = id;
  page = p;
  img_H = iH;
  img_B = iB;
}

boolean BTN::select(int sX, int sY) {
  if (actif && visible && sX > x && sX < x + dX && sY > y && sY < y + dY) {
    if (valide && !(appuye && exclusif))
      appuye = ! appuye;
    affiche(appuye);
    if (valide) {
      while (myTouch.dataAvailable()) // attend que l'appui sur l'cran cesse
        myTouch.read();
      if (retourHaut) {
        appuye = false;
        affiche(false);
      }
      else {
        if (exclusif) {
          PtrComp * compExc = compDeb;
          for (int i = 0; i < nbComp; i++) { // liste des composants visuels
            if (compExc->ptrB->groupe == groupe && compExc->ptrB != this) {
              compExc->ptrB->appuye = false;
              compExc->ptrB->affiche(false);
            }
            compExc = compExc->suiv;
          }
        }
      }
      if (surAppui != NULL)
        surAppui(indX/*, appuye*/);
      return true;
    }
    else
      return false;
  }
  else
    return false;
}

void BTN::efface() {
  visible = false;
  myGLCD.setColor(clFond);
  myGLCD.fillRect(x, y, x + dX, y + dY);
}

void  BTN::affiche() {
  affiche(appuye);
}

void BTN::affiche(boolean app) { // app = true -> bouton appuy
  if (visible) {
    if (app)
      myGLCD.drawTransparentBitmap(x, y, 16, 16, img_B);
    else
      myGLCD.drawTransparentBitmap(x, y, 16, 16, img_H);
  }
}

/*-----------------------------------------------------*/
/*                           INCDEC                           */
/*-----------------------------------------------------*/

IncDec::IncDec(int a, int b, byte o, byte g, byte p) :
  amplPlus(0, 0, BtnPlus_H, BtnPlus_B, true, 1 + 2 * g, p),
  amplMoins(0, 0, BtnMoins_H, BtnMoins_B, true, 2 + 2 * g, p) {
  tag = 0x13;
  x = a;
  y = b;
  switch (o) {
    case VERT:
      dX = 31;
      dY = 63;
      amplPlus.x = x + 8;
      amplPlus.y = y + 3;
      amplMoins.x = x + 8;
      amplMoins.y = y + 45;
      espTxt = dX - 1;
      xTxt1 = 1;
      xTxt2 = 1;
      yTxt1 = 23;
      yTxt2 = 33;
      break;
    case HORZ:
      dX = 78;
      dY = 19;
      amplPlus.x = x + 3;
      amplPlus.y = y + 2;
      amplMoins.x = x + 60;
      amplMoins.y = y + 2;
      espTxt = 41;
      xTxt1 = 20;
      xTxt2 = 20;
      yTxt1 = 2;
      yTxt2 = 11;
      break;
  }
  visible = true;
  actif = true;
  page = p;
}

void IncDec::efface() {
  visible = false;
  myGLCD.setColor(clFond);
  myGLCD.fillRect(x, y, x + dX, y + dY);
}

void IncDec::chgtTxt(byte ind, String txt) {
  if (visible) {
    myGLCD.setBackColor(VGA_SILVER);
    myGLCD.setColor(VGA_SILVER);
    switch (ind) {
      case 1:
        txt1 = txt;
        myGLCD.fillRect(x + xTxt1, y + yTxt1, x + xTxt1 + espTxt, y + yTxt1 + fontDimY - 1);
        myGLCD.setColor(VGA_BLACK);
        myGLCD.print(txt1, x + xTxt1 + (espTxt - txt1.length()*fontDimX) / 2, y + yTxt1);
        break;
      case 2:
        txt2 = txt;
        myGLCD.fillRect(x + xTxt2, y + yTxt2, x + xTxt2 + espTxt, y + yTxt2 + fontDimY - 1);
        myGLCD.setColor(VGA_BLACK);
        myGLCD.print(txt2, x + xTxt2 + (espTxt - txt2.length()*fontDimX) / 2, y + yTxt2);
        break;
    }
  }
}

void IncDec::affiche() {
  if (visible) {
    myGLCD.setColor(VGA_SILVER);
    myGLCD.setBackColor(VGA_SILVER);
    myGLCD.fillRoundRect(x, y, x + dX, y + dY);
    chgtTxt(1, txt1);
    chgtTxt(2, txt2);
    amplMoins.affiche();
    amplPlus.affiche();
  }
}

/*-----------------------------------------------------*/
/*                    OSCILLOSCOPE                     */
/*-----------------------------------------------------*/

Oscilloscope::Oscilloscope(byte a) :
		indexZero(POS_H - 2, POS_V, NBPT_Y, 50, 'E', 0xFC),
		indexTrig(POS_H + NBPT_X + 2, POS_V, NBPT_Y, 50, 'O', 0xFC),
		amplV(0, 30, VERT, 0, 0xFC),
		baseT(50, POS_V + NBPT_Y + 3, HORZ, 1, 0xFC),
		btnAC(2, BLOC_ACDC + 2, 28, 20, "AC", NULL, false, 0xFC),
		btnGND(2, BLOC_ACDC + 25, 28, 20, "GND", NULL, false, 0xFC),
		btnDC(2, BLOC_ACDC + 48, 28, 20, "DC", NULL, false, 0xFC),
		btnTrig(160, POS_V + NBPT_Y + 3, 60, 18, "Trig", Trig_M, true, 0xFC) {
  pinMode(CORR_ZERO, OUTPUT);
  pinMode(MUX_ACDC_0, OUTPUT);
  pinMode(MUX_ACDC_1, OUTPUT);
  pinMode(MUX_10kN_0, OUTPUT);
  pinMode(MUX_10kN_1, OUTPUT);
  pinMode(MUX_k125_0, OUTPUT);
  pinMode(MUX_k125_1, OUTPUT);
  tag = 0x09;
  btnAC.groupe = 1;
  btnGND.groupe = 1;
  btnDC.groupe = 1;
  btnAC.exclusif = true;
  btnGND.exclusif = true;
  btnDC.exclusif = true;
  btnGND.appuye = true;
	indxV = 5;
	changement(1);
  indxH = 5;
  txtH = paramBT[indxH].info;
  amplV.txt2 = "/div";
  baseT.txt2 = "/div";
  visible = false;
  actif = true;
  page = 0xFC;
  zeroAff = ZERO_CAN + NBPT_Y / 2;
  horsSynchro = false;
  btnAC.surAppui = &modeAC;
  btnGND.surAppui = &modeGND;
  btnDC.surAppui = &modeDC;
  amplV.amplPlus.surAppui = &changement;
  amplV.amplMoins.surAppui = &changement;
  baseT.amplPlus.surAppui = &changement;
  baseT.amplMoins.surAppui = &changement;
  btnTrig.surAppui = &chgtTrig;
	modeGND(true);
}

void Oscilloscope::init() {
	myGLCD.drawTransparentBitmap(2, 104, 316, 32, Osc_ChP);
	delay(1000);
	myGLCD.setColor(clFond);
	for (int i = 0; i < scrDimX; i++) {
		myGLCD. drawLine(i, 104, i, 136);
		delay(4);
	}
}

void Oscilloscope::modeAC(boolean) {
  digitalWrite(MUX_ACDC_0, LOW);
  digitalWrite(MUX_ACDC_1, LOW);
}

void Oscilloscope::modeGND(boolean) {
  digitalWrite(MUX_ACDC_0, LOW);
  digitalWrite(MUX_ACDC_1, HIGH);
}

void Oscilloscope::modeDC(boolean) {
  digitalWrite(MUX_ACDC_0, HIGH);
  digitalWrite(MUX_ACDC_1, LOW);
}

void Oscilloscope::changement(byte i/*, boolean a*/) {
  switch (i) {
    case 1: // Incrmentation axeV
      if (indxV < sizeStrV - 1) {
        indxV++;
        txtV = calVert[indxV];
        chgtOK = true;
      }
      break;
    case 2: // Dcrmentation axeV
      if (indxV > 0) {
        indxV--;
        txtV = calVert[indxV];
        chgtOK = true;
      }
      break;
    case 3: // Incrmentation axeH
      if (indxH < sizeStrH - 1) {
        indxH++;
        txtH = paramBT[indxH].info;
        chgtOK_BT = true;
      }
      break;
    case 4: // Dcrmentation axeH
      if (indxH > 0) {
        indxH--;
        txtH = paramBT[indxH].info;
        chgtOK_BT = true;
      }
      break;
  }
  switch (indxV % 3) { // Commutation des gains 1 - 2 - 5
    case 0: // Gain = 1
      digitalWrite(MUX_k125_0, LOW);
      digitalWrite(MUX_k125_1, LOW);
      break;
    case 1: // Gain = 2
      digitalWrite(MUX_k125_0, HIGH);
      digitalWrite(MUX_k125_1, LOW);
      break;
    case 2: // Gain = 5
      digitalWrite(MUX_k125_0, LOW);
      digitalWrite(MUX_k125_1, HIGH);
      break;
  }
  if (indxV >= 6) { // Gain = 1
    digitalWrite(MUX_10kN_0, LOW);
    digitalWrite(MUX_10kN_1, HIGH);
  } else if (indxV >= 3) { // Gain = 10
    digitalWrite(MUX_10kN_0, HIGH);
    digitalWrite(MUX_10kN_1, LOW);
  } else { // Gain = 100
    digitalWrite(MUX_10kN_0, LOW);
    digitalWrite(MUX_10kN_1, LOW);
  }
}

void Oscilloscope:: chgtTrig(boolean a) {
  modeTrig++;
  if (modeTrig == 3)
    modeTrig = 0;
  chgtOK = true;
}

void Oscilloscope::montre() {
  TCCR2B &= 0B11111000; // Compteur arrt
  visible = true;
  myGLCD.fillScr(clFond);
  myGLCD.setColor(VGA_SILVER);
  myGLCD.fillRoundRect(0, BLOC_ACDC, 31, BLOC_ACDC + 71);
  ongletAppel = ongletActif; // mmorise de quel onglet vient l'appel
  ongletActif = 0xFC; // onglet de l'oscilloscope
  installeComp(ongletActif);
  for (int i = 0; i < NBPT_X + 1; i++)
    eff[i] = 128;
  cadre();
  cli();          // disable global interrupts
  ADMUX = B01100000; // ref = VCC, entre A0, justification Left
  ADCSRA = B11000110; // Freq ADC = 16 MHz/64 pour 2mS/carreau
  TCCR2A = 0;     // set entire TCCR2A register to 0
  TCCR2B = 0;     // same for TCCR2B
  TCCR2A |= (1 << WGM21);  // Set to CTC Mode
  TIMSK2 |= (1 << OCIE2A);  //Set interrupt on compare match
  OCR2A = paramBT[indxH].compDiv; // Valeur d'arrt pour la comparaison
  TCCR2B |= paramBT[indxH].preDiv;
  indR = 0;
  sei();
}

void Oscilloscope::affiche() {
}

void Oscilloscope::grad() {
  myGLCD.setColor(CL_GRAD);
  for (byte i = 1; i < 10; i++)
    myGLCD.drawLine(POS_H + i * GRAD, POS_V, POS_H + i * GRAD, POS_V + NBPT_Y);
  for (byte i = 1; i < 8; i++)
    myGLCD.drawLine(POS_H , POS_V + i * GRAD, POS_H + NBPT_X, POS_V + i * GRAD);
}

void Oscilloscope::cadre() {
  myGLCD.setColor(clFond);
  myGLCD.fillRect(POS_H + 1, POS_V + 1, POS_H + NBPT_X - 1, POS_V + NBPT_Y - 1);
  grad();
  myGLCD.setColor(CL_CADRE);
  myGLCD.drawRect(POS_H, POS_V, POS_H + NBPT_X, POS_V + NBPT_Y);
  myGLCD.drawRect(POS_H - 1, POS_V - 1, POS_H + 1 + NBPT_X, POS_V + 1 + NBPT_Y);
}

void Oscilloscope::majParams() {
  if (chgtOK) {
    amplV.chgtTxt(1, txtV);
    switch (modeTrig) {
      case 0:
        btnTrig.icone = Trig_A;
        break;
      case 1:
        btnTrig.icone = Trig_M;
        break;
      case 2:
        btnTrig.icone = Trig_D;
        break;
    }
    btnTrig.affiche();
    chgtOK = false;
  }
  if (chgtOK_BT) {
    baseT.chgtTxt(1, txtH);
    cadre();
    chgtOK_BT = false;
  }
}

byte Oscilloscope::affCrb(volatile int crb[], char typeAff, int d, byte n) {
  if (modeTrig != 0) { // Si pas en mode auto
    if (d > 2 * NBPT_X && ! horsSynchro) {
      horsSynchro = true;
      cadre(); // On retrace le cadre pour effacer l'ancienne courbe
    }
    if (d > NBPT_X)
      return 0xFF;
    else
      horsSynchro = false;
  }
  majParams(); // Prend en compte les nouveaux paramtres d'affichage
	if (btnGND.appuye) {
		fauxZero = 0;
		for (byte i = 10; i < 60; i++)
			fauxZero += crb[i];
		fauxZero = fauxZero / 50.0 - ZERO_CAN - NBPT_Y / 2 + indexZero.curPos; // Correction du faux Zro (Ampli Op et biais de analogWrite())
	}
  analogWrite(CORR_ZERO, - indexZero.pos * 0xFF / 100.0); // La correction de zro est fonction de la position du curseur "indexZero"
  byte err = 0;
  int j;
  switch (typeAff) {
    case 'N': // Normal, tous les points sont affichs
      j = 1;
      for (int i = d; i < NBPT_X + d; i++) {
        myGLCD.setColor(clFond);
        if (eff[j] != eff[j - 1]) // Si la valeur actuelle est diffrente de la valeur prcdente
          myGLCD.drawLine(j + POS_H - 1,  - eff[j - 1], j + POS_H - 1,  - eff[j]); // ligne verticale
        else
          myGLCD.drawLine(j + POS_H - 1 - 1,  -  eff[j], j + POS_H - 1,  - eff[j]); // ligne horizontale
        myGLCD.setColor(CL_COURBE);
        crb[i] -= zeroAff + fauxZero;
        if (-crb[i] < POS_V) { // dpassement positif
          crb[i] = -POS_V;
          err = 1;
        } else if (-crb[i] > POS_V + NBPT_Y) { // dpassement ngatif
          crb[i] = - POS_V - NBPT_Y;
          err = -1;
        }
        if (crb[i] != crb[i - 1]) // Si la valeur actuelle est diffrente de la valeur prcdente
          myGLCD.drawLine(i + POS_H - d,  - crb[i - 1], i + POS_H - d,  - crb[i]); // ligne verticale
        else
          myGLCD.drawLine(i + POS_H - 1 - d,  - crb[i], i + POS_H - d,  - crb[i]); // ligne horizontale
        j++;
      }
      grad(); // Retraage des graduations
      for (int i = -1; i < NBPT_X; i++)
        eff[i + 1] = crb[i + d];
      break;
    case 'P': // Un point sur n du fichier est affich et est espac de n pixels du prcdent
      for (int i = 0; i < NBPT_X + 1; i += n)
        crb[i] += zeroAff + fauxZero; // Intgration du dcalage de zro
      if (crb[d] < -4 * GRAD) { // dpassement positif
        crb[d] = -4 * GRAD;
        err = 1;
      } else if (crb[d] > 4 * GRAD) { // dpassement ngatif
        crb[d] = 4 * GRAD;
        err = -1;
      }
      for (int i = d; i < NBPT_X + d; i += n) {
        myGLCD.setColor(clFond);
        myGLCD.drawLine(i + POS_H - d, ZERO_CAN - eff[i], (i + n) + POS_H - d, ZERO_CAN - eff[i + n]); // ligne horizontale
        myGLCD.setColor(CL_COURBE);
        if (crb[i + n] < -4 * GRAD) { // dpassement positif
          crb[i + n] = -4 * GRAD;
          err = 1;
        } else if (crb[i + n] > 4 * GRAD) { // dpassement ngatif
          crb[i + n] = 4 * GRAD;
          err = -1;
        }
        myGLCD.drawLine(i + POS_H - d, ZERO_CAN - crb[i], (i + n) + POS_H - d, ZERO_CAN - crb[i + n]); // ligne horizontale
      }
      grad(); // Retraage des graduations
      for (int i = d; i < NBPT_X + 1 + d; i += n)
        eff[i] = crb[i];
      break;
    case 'L': // Loupe : // Tous les points sont affichs et sont espac de n pixels entre eux
      for (int i = d; i < NBPT_X / n + 1 + d; i++)
        crb[i] -= zeroAff + fauxZero;
      j = 0;
      for (int i = d; i < NBPT_X / n + d; i++) {
        myGLCD.setColor(clFond);
        myGLCD.drawLine(n * j + POS_H, - eff[j], n * (j + 1) + POS_H, - eff[j + 1]);
        myGLCD.setColor(CL_COURBE);
        if (-crb[i] < POS_V) { // dpassement positif
          crb[i] = -POS_V;
          err = 1;
        } else if (-crb[i] > POS_V + NBPT_Y) { // dpassement ngatif
          crb[i] = - POS_V - NBPT_Y;
          err = -1;
        }
        myGLCD.drawLine(n * (i - d) + POS_H, - crb[i], n * (i + 1 - d) + POS_H, - crb[i + 1]);
        j++;
      }
      grad(); // Retraage des graduations
      j = 0;
      for (int i = d; i < NBPT_X / n + 1 + d; i++) {
        eff[j] = crb[i];
        j++;
      }
      break;
  }
  return err;
}

int Oscilloscope::synchro() {
  int i = 1; // Problme non rsolu si dpart sur i = 0
  int niv;
	int seuilTrig = scrDimY - indexTrig.curPos;
  switch (modeTrig) {
    case 0: // Auto
    case 1: // Front montant
      while (res[i] > (seuilTrig) && i < NBPT_X)
        i++;
			i += 1; // pseudo hystrsis ??
      while (res[i] < (seuilTrig) && i < NBPT_X)
        i++;
      niv = res[i - 1];
			i += 1; // pseudo hystrsis ??
      while (res[i] < (niv + 5) && i < NBPT_X)
        i++;
      break;
    case 2: // front descendant
      while (res[i] < (seuilTrig) && i < NBPT_X)
        i++;
			i += 1; // pseudo hystrsis ??
      while (res[i] > (seuilTrig) && i < NBPT_X)
        i++;
      niv = res[i - 1];
			i += 1; // pseudo hystrsis ??
      while (res[i] > niv - 5 && i < NBPT_X)
        i++;
      break;
  }
  return i;
}

void Oscilloscope::majCrb() {
  if (actif && visible)
    if (indR >= 2 * NBPT_X) {
      TCCR2B &= 0B11111000; // Compteur arrt
      affCrb(res, paramBT[indxH].typeAff, synchro(), paramBT[indxH].sautPts);
      OCR2A = paramBT[indxH].compDiv; // Valeur d'arrt pour la comparaison
      TCCR2B |= paramBT[indxH].preDiv;
      ADCSRA = paramBT[indxH].freqEch; // Freq ADC = 16 MHz/64 pour 2mS/carreau
      indR = 0; // Rinitialisation du compteur de lecture
    }
}

boolean Oscilloscope::select(int sX, int sY) {
  if (actif && visible)
    if (sX > POS_H + MARGE && sX < POS_H + NBPT_X - MARGE && sY > POS_V + MARGE && sY < POS_V + NBPT_Y - MARGE) {
      if (surAppui != NULL)
        surAppui(true);
    }
}

