#ifndef Oscilloscope_h
#define Oscilloscope_h

#include <Arduino.h>
#include <BaseComp.h>

#define GRAD 27
#define NBPT_X 10 * GRAD
#define NBPT_Y 8 * GRAD
#define CL_CADRE VGA_LIME
#define CL_GRAD  VGA_YELLOW
#define CL_COURBE VGA_WHITE
#define CL_FOND VGA_BLACK
#define POS_H 41
#define POS_V  1
#define MARGE 20
#define VERT   0
#define HORZ  1
#define NBPT 810

extern unsigned int BtnPlus_H[0x100];
extern unsigned int BtnPlus_B[0x100];
extern unsigned int BtnMoins_H[0x100];
extern unsigned int BtnMoins_B[0x100];
extern unsigned int Trig_A[0x100];
extern unsigned int Trig_M[0x100];
extern unsigned int Trig_D[0x100];
extern unsigned int Osc_ChP[0x2780];

class BTN : public CompBase {
  protected:
    void dessineBtn(int, int, boolean);
  public:
    BTN() {};
    BTN(int, int, bitmapdatatype, bitmapdatatype, boolean, byte, byte = 0);
    ~BTN() {};
    byte indX;
    boolean exclusif;
    bitmapdatatype img_H;
    bitmapdatatype img_B;
    boolean retourHaut;
    void (* surAppui)(byte/*, boolean*/);
    virtual void affiche();
    void affiche(boolean); // dessine le composant
    void efface();
    virtual boolean select(int, int);
};

class IncDec : public CompBase {
  private:
    byte espTxt;
    byte xTxt1, xTxt2;
    byte yTxt1, yTxt2;
  public:
    BTN amplPlus;
    BTN amplMoins;
    String txt1, txt2;
    IncDec(int, int, byte, byte, byte = 0);
		void chgtTxt(byte, String);
    virtual void affiche();
    void efface();
};

class Oscilloscope : public CompBase {
  private:
		static void modeAC(boolean);
		static void modeGND(boolean);
		static void modeDC(boolean);
    static void changement(byte);
		static void chgtTrig(boolean);
		void majParams();
    byte ongletAppel; // mmorisation de l'onglet qui a appel
    int zeroAff;
		float fauxZero;
		boolean horsSynchro;
    IncDec amplV;
    IncDec baseT;
    Index indexZero;
    Index indexTrig;
    Bouton btnAC;
    Bouton btnGND;
    Bouton btnDC;
		Bouton btnTrig;
  public:
    Oscilloscope(byte);
		void init();
    void grad();
    void cadre();
		int synchro();
    byte affCrb(volatile int crb[], char, int, byte);
		void majCrb();
    void montre();
    virtual void affiche();
    virtual boolean select(int sX, int sY);
};

#endif