#include <Arduino.h>
#include <ServoM.h>

volatile unsigned long preLoad;
volatile byte PORT_B;
volatile byte PORT_D;
volatile unsigned long t0;
volatile boolean top = false;
byte nbServoM = 0;
PtrServoM * servoM_Deb = NULL; // lment du dbut de la liste
PtrServoM * servoM_Cour = new PtrServoM; // pointeur d'lment de la liste

void (* surModif)(); // pointeur sur une fonction d'action

void initServoM() { // initialize Timer1
  preLoad = 65536 - F_CPU / (256 * 50); // Le facteur de prdivision est dfini par la frquence du CPU
  cli();         // disable global interrupts
  TCCR1A = 0;    // set entire TCCR1A register to 0
  TCCR1B = 0;    // set entire TCCR1A register to 0
  TIMSK1 |= (1 << TOIE1);  // enable Timer1 overflow interrupt:
  // Preload with value 3036
  //use 64911 for 100Hz
  //use 64286 for 50 Hz
  //use 34286 for 2 Hz
  TCNT1 = preLoad;
  // Set CS10 bit so timer runs at clock speed: (no prescaling)
  TCCR1B |= (1 << CS12); // Sets bit CS12 in TCCR1B
  // This is achieved by shifting binary 1 (0b00000001)
  // to the left by CS12 bits. This is then bitwise
  // OR-ed into the current value of TCCR1B, which effectively set
  // this one bit high. Similar: TCCR1B |= _BV(CS12);
  //  or: TCCR1B= 0x04;
  sei();  // enable global interrupts:
}

ISR(TIMER1_OVF_vect) {
  t0 = micros();
  servoM_Cour = servoM_Deb;
  for (int i = 0; i < nbServoM; i++) { // boucle qui permet de voir si un servoM est actif
    if (servoM_Cour->ptrSM->actif) { // Mise au niveau haut des sorties concernes
      if (servoM_Cour->ptrSM->pin < 8) // Les broches de 2  7
        PORT_D |= (1 << servoM_Cour->ptrSM->pin);
      else
        PORT_B |= (1 << (servoM_Cour->ptrSM->pin - 8)); // Les broches 8  13
      servoM_Cour->ptrSM->vu = false;
      if (servoM_Cour->ptrSM->vitesse != 0) { // prise en compte d'un changement  vitesse constante
        servoM_Cour->ptrSM->duree += servoM_Cour->ptrSM->sens * servoM_Cour->ptrSM->incDuree; // Accroissement de l'angle command
        if (servoM_Cour->ptrSM->sens * servoM_Cour->ptrSM->duree >= servoM_Cour->ptrSM->sens * servoM_Cour->ptrSM->dureeCmd) {
          servoM_Cour->ptrSM->vitesse = 0;
          servoM_Cour->ptrSM->finSeq = true;
        }
      }
    }
    servoM_Cour = servoM_Cour->suiv;
  }
  PORTB = PORT_B; // Je globalise les diffrents ports pour qu'ils commutent tous en mme temps
  PORTD = PORT_D; // Idem ci-dessus
  TCNT1 = preLoad; // rechargement du compteur pour qu'il reparte pour un tour
  top = true;
}

void scruteServoM() { // scrutation d'vnements
  if (top) {
    top = false;
    servoM_Cour = servoM_Deb;
    unsigned long t1 = micros() - t0;
    while (t1 < T_MAX + 50) {
      for (int i = 0; i < nbServoM; i++) { // boucle testant la dure de chaque instance active
        if (servoM_Cour->ptrSM->actif) {
          if (! servoM_Cour->ptrSM->vu && t1 >= servoM_Cour->ptrSM->duree) {
            if (servoM_Cour->ptrSM->pin < 8)
              PORTD &= ~(1 << servoM_Cour->ptrSM->pin);
            else
              PORTB &= ~(1 << servoM_Cour->ptrSM->pin - 8);
            servoM_Cour->ptrSM->vu = true;
          }
        }
        servoM_Cour = servoM_Cour->suiv;
      }
      t1 = micros() - t0;
    }
    if (surModif != NULL)
      surModif();
  }
}

/*-----------------------------------------------------*/
/*                         SERVO_M                          */
/*-----------------------------------------------------*/

void ServoM::setTime(int t) {
  if (t < tMin)
    duree = tMin;
  else if (t > tMax)
    duree = tMax;
  else
    duree = t;
  actif = true;
}

void ServoM::setAngle(int ang, int vit) { // ang en  et vit en /s
  angle = ang;
  vitesse = vit;
  if (vit == 0)
		setTime(map(ang, -90, 90, tMin, tMax));
  else {
		dureeCmd = map(ang, -90, 90, tMin, tMax);
    if (dureeCmd > duree)
      sens = 1;
    else if (dureeCmd < duree)
      sens = -1;
    incDuree = 0.222 * vitesse;
    finSeq = false;
  }
}