#include <Arduino.h>
#include <UTM-LatLon.h>

float lat; // latitude en degrés décimaux
float lon; // longitude en degrés décimaux
float utm_N; // Northing en mètres
float utm_E; // Easting en mètres
String utm_Z; // zone
float k0 = 0.9996f;
float equR = 6378137; // Rayon de l'ellipsoïde valeur WGS84
float exc2 = 0.00669438; // Excentricité de l'ellipsoïde valeur WGS84
float excBis2 = 0.0067394968;


float toRadians(float deg) {
  return deg * PI / 180;
}

float toDegrees(float rad) {
  return rad * 180 / PI;
}

String dDecToDMS(float t, boolean neg) {
  String formatDeg = F("%03d\xA7 %02d' %02d\".%02d"); // format d'angle sexagésimal pour l'affichage (mis en mémoire flash)
  String sgn = "";
  if (t < 0) {
    if (!neg)
      t = 360+t;
    else {
      sgn = "-";
      t = -t;
    }
  }
  int h = floor(t);
  t = (t - h) * 60;
  byte m = floor(t);
  t = (t - m) * 60;
  byte s = floor(t);
  t = (t - s) * 100;
  byte fs = floor(t);
  char str[17]; // stockage de la chaïne complète
  sprintf(str, formatDeg.c_str(), h, m, s, fs);
  return sgn+str;
}

String CodeZone(float lat) {
 String sZ = F("CDEFGHJKLMNPQRSTUVWXZ"); // Les différentes zones
 byte i;
  if (lat > 80 || lat < -80)
    return "Z";
  else {
    for (i = 0; i < 20; i++)
      if (lat < -72 + 8 * i)
        break;
  }
  return sZ.substring(i, i + 1);
}

/*char CodeZone(float lat) {
 char sZ[] = "CDEFGHJKLMNPQRSTUVWXZ"; // Les différentes zones
 byte i;
  if (lat > 80 || lat < -80)
    return 'Z';
  else {
    for (i = 0; i < 20; i++)
      if (lat < -72 + 8 * i)
        break;
  }
  return sZ[i];
}*/

void LatLonVersUTM(float lat, float lon) {
  float orgLon, lonTmp;
  float latRd, lonRd, orgLonRd;
  float N, T, C, A, M;
  int noZone;
  lonTmp = lon - round(lon / 360);
  latRd = toRadians(lat);
  lonRd = toRadians(lon);
  noZone = (int) (round((lonTmp + 180) / 6 - 0.5) + 1);
  if (lat >= 56 && lat < 64 && lonTmp >= 3 && lonTmp < 12)
    noZone = 33;
  else if (lat >= 72 && lat < 84)
    if (lonTmp >= 0 && lonTmp < 9)
      noZone = 31;
    else if (lonTmp >= 9 && lonTmp < 21)
      noZone = 33;
    else if (lonTmp >= 21 && lonTmp < 33)
      noZone = 35;
    else if (lonTmp >= 33 && lonTmp < 42)
      noZone = 37;
  utm_Z = String(CodeZone(lat)) + noZone;
  orgLon = (noZone - 1) * 6 - 180 + 3;
  orgLonRd = toRadians(orgLon);
  N = equR / sqrt(1 - exc2 * pow(sin(latRd), 2));
  T = pow(tan(latRd), 2);
  C = excBis2 * pow(cos(latRd), 2);
  A = cos(latRd) * (lonRd - orgLonRd);
	float exc2S = pow(exc2, 2);	
	float exc2C = exc2 * exc2S;	
  M = equR * ((1 - exc2 / 4 - 3 * exc2S / 64 - 5 * exc2C / 256) * latRd
              - sin(2 * latRd) * (384 * exc2 + 96 * exc2S + 45 * exc2C) / 1024
              + sin(4 * latRd) * (60 * exc2S + 45 * exc2C) / 1024
              - sin(6 * latRd) * (35 * exc2C / 3072));
  utm_E = k0 * N * (A + (1 - T + C) * pow(A, 3) / 6 + (5 - 18 * pow(T, 3) + 72 * C - 58 * excBis2) * pow(A, 5) / 120) + 500000;
  utm_N = k0 * (M + N * tan(latRd) * (pow(A, 2) / 2 + (5 - T + 9 * C + 4 * pow(C, 2)) * pow(A, 4) / 24
                                      + (61 - 58 * pow(T, 3) + 600 * C - 330 * excBis2) * pow(A, 6) / 720));
  if (lat < 0)
    utm_N = utm_N + 10000000;
}

void utmVersLatLon(float utm_E, float utm_N, String utm_Z) {
  float OrgLon;
  float el;
  float x, y;
  int noZ;
  float m, mu, Phi1rd;
  float n1, t1, c1, r1, d;
  el = (1 - sqrt(1 - exc2)) / (1 + sqrt(1 - exc2));
  x = utm_E - 500000;
  y = utm_N;
  utm_Z.toUpperCase();
  if (int(utm_Z.charAt(0)) - 78 < 0) { // 78 = valeur ASCII de 'N'
    if (y > 0)
      y = utm_N - 10000000;
  }
  noZ = utm_Z.substring(1, 3).toInt();
  OrgLon = (noZ - 1) * 6 - 180 + 3; // +3 situe l'origine au milieu de la zone
  m = y / k0;
  mu = m / (equR * (1 - exc2 / 4 - 3 * pow(exc2, 2) / 64 - 5 * pow(exc2, 3) / 256));
  Phi1rd = mu + (3 * el / 2 - 27 * pow(el, 3) / 32) * sin(2 * mu)
           + (21 * pow(el, 2) / 16 - 55 * pow(el, 4) / 32) * sin(4 * mu)
           + (151 * pow(el, 3) / 96) * sin(6 * mu);
  n1 = equR / sqrt(1 - exc2 * pow(sin(Phi1rd), 2));
  t1 = pow(tan(Phi1rd), 2);
  c1 = excBis2 * pow(cos(Phi1rd), 2);
  r1 = equR * (1 - exc2) / pow(1 - exc2 * pow(sin(Phi1rd), 2), 1.5);
  d = x / (n1 * k0);
  lat = Phi1rd - n1 * tan(Phi1rd) / r1 * (pow(d, 2) / 2 - (5 + 3 * t1 + 10 * c1 - 4 * pow(c1, 2) - 9 * excBis2) * pow(d, 4) / 24
                                          + (61 + 90 * t1 + 298 * c1 + 45 * pow(t1, 2) - 252 * excBis2 - 3 * pow(c1, 2)) * pow(d, 6) / 720);
  lat = toDegrees(lat);
  lon = (d - (1 + 2 * t1 + c1) * pow(d, 3) / 6 + (5 - 2 * c1 + 28 * t1 - 3 * pow(c1, 2) + 8 * excBis2 + 24 * pow(t1, 2)) * pow(d, 5) / 120) / cos(Phi1rd);
  lon = OrgLon + toDegrees(lon);
}
